from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class GraphicFrameLocking(Serialisable):
    noGrp: Any
    noDrilldown: Any
    noSelect: Any
    noChangeAspect: Any
    noMove: Any
    noResize: Any
    extLst: Any
    def __init__(
        self,
        noGrp: Any | None = ...,
        noDrilldown: Any | None = ...,
        noSelect: Any | None = ...,
        noChangeAspect: Any | None = ...,
        noMove: Any | None = ...,
        noResize: Any | None = ...,
        extLst: Any | None = ...,
    ) -> None: ...

class NonVisualGraphicFrameProperties(Serialisable):
    tagname: str
    graphicFrameLocks: Any
    extLst: Any
    def __init__(self, graphicFrameLocks: Any | None = ..., extLst: Any | None = ...) -> None: ...

class NonVisualGraphicFrame(Serialisable):
    tagname: str
    cNvPr: Any
    cNvGraphicFramePr: Any
    __elements__: Any
    def __init__(self, cNvPr: Any | None = ..., cNvGraphicFramePr: Any | None = ...) -> None: ...

class GraphicData(Serialisable):
    tagname: str
    namespace: Any
    uri: Any
    chart: Any
    def __init__(self, uri=..., chart: Any | None = ...) -> None: ...

class GraphicObject(Serialisable):
    tagname: str
    namespace: Any
    graphicData: Any
    def __init__(self, graphicData: Any | None = ...) -> None: ...

class GraphicFrame(Serialisable):
    tagname: str
    nvGraphicFramePr: Any
    xfrm: Any
    graphic: Any
    macro: Any
    fPublished: Any
    __elements__: Any
    def __init__(
        self,
        nvGraphicFramePr: Any | None = ...,
        xfrm: Any | None = ...,
        graphic: Any | None = ...,
        macro: Any | None = ...,
        fPublished: Any | None = ...,
    ) -> None: ...

class GroupShape(Serialisable):
    nvGrpSpPr: Any
    nonVisualProperties: Any
    grpSpPr: Any
    visualProperties: Any
    pic: Any
    __elements__: Any
    def __init__(self, nvGrpSpPr: Any | None = ..., grpSpPr: Any | None = ..., pic: Any | None = ...) -> None: ...



import selectors
import sys

from . import base_events

if sys.version_info >= (3, 7):
    __all__ = ("BaseSelectorEventLoop",)
else:
    __all__ = ["BaseSelectorEventLoop"]

class BaseSelectorEventLoop(base_events.BaseEventLoop):
    def __init__(self, selector: selectors.BaseSelector | None = ...) -> None: ...



"""Type system extensions for use with pytype."""
import dataclasses
import typing
from typing import Any, Callable, Dict, Generic, Text, Type, TypeVar

# pylint: disable=g-import-not-at-top
try:
  from typing import Protocol
except ImportError:
  # typing.Protocol was added in python 3.8.
  from typing_extensions import Protocol
# pylint: enable=g-import-not-at-top

T = TypeVar('T')


class Dataclass(Protocol[T]):
  """Protocol that matches any dataclass (or instance thereof).

  Can be used to match any dataclass. Example (modulo pytype bugs):
  @dataclasses.dataclass
  class Foo:
    x: str
    y: int

  @dataclasses.dataclass
  class Bar:
    x: str
    y: str

  class Baz:
    x: str
    y: int

  def foo(item: Dataclass):
    pass

  def bar(item: Dataclass[str]):
    pass

  def baz(item: Dataclass[Union[int, str]]):
    pass

  foo(Foo(x='yes', y=1))     # ok
  foo(Bar(x='yes', y='no'))  # ok
  foo(Baz(x='yes', y=1))     # error, not a dataclass

  bar(Foo(x='yes', y=1))     # error, has a non-str field
  bar(Bar(x='yes', y='no'))  # ok
  bar(Baz(x='yes', y=1))     # error, not a dataclass

  baz(Foo(x='yes', y=1))     # ok
  baz(Bar(x='yes', y='no'))  # ok
  baz(Baz(x='yes', y=1))     # error, not a dataclass

  The only way to identify a dataclass is to test for the presence of the
  __dataclass_fields__ member; that is what dataclasses.is_dataclass uses:
  https://github.com/python/cpython/blob/3.7/Lib/dataclasses.py#L1036.
  """

  __dataclass_fields__: Dict[str, 'dataclasses.Field[T]']


if typing.TYPE_CHECKING:

  _GenericCallable = TypeVar('_GenericCallable', bound=Callable[..., Any])

  class Decorator:
    """A type annotation for decorators that do not change signatures.

    This is a stand-in for using `Callable[[T], T]` to represent a decorator.

    Given a decorator function, which takes in a callable and returns a callable
    with the same signature, apply this class as a decorator to that function.
    This can also be used for decorator factories.

    Examples:

    Plain decorator (decorator matches Callable[[T], T]):

    >>> @pytype_extensions.Decorator
    ... def MyDecorator(func):
    ...   def wrapper(...):
    ...     ...
    ...   return wrapper

    Decorator factory (factory matches Callable[..., Callable[[T], T]]):

    >>> def MyDecoratorFactory(foo: int) -> pytype_extensions.Decorator:
    ...   @pytype_extensions.Decorator
    ...   def MyDecorator(func):
    ...     def Wrapper(*args, **kwargs):
    ...       return func(foo, *args, **kwargs)
    ...     return Wrapper
    ...   return MyDecorator

    Note for the above example: the return type annotation (first line) is the
    most important one; it indicates to callers that MyDecoratorFactory is
    returning a decorator. The "@pytype_extensions.Decorator" annotation (second
    line) indicates to pytype that MyDecorator is a Decorator; without it, you
    would need to add "pytype: disable=bad-return-type" on the final line.

    This class only exists at build time, for typechecking. At runtime, the
    'Decorator' member of this module is a simple identity function (see below).

    More information: pytype-decorators

    Shortlink: pytype_extensions.Decorator
    """
    # pylint: disable=line-too-long, unused-argument

    def __init__(self, decorator: Callable[[_GenericCallable], _GenericCallable]):
      ...

    def __call__(self, func: _GenericCallable) -> _GenericCallable:
      ...  # pytype: disable=bad-return-type

else:
  # At runtime, Decorator is a simple identify function that returns its input.
  Decorator = lambda d: d


def assert_type(*unused_args):
  """Prevent runtime errors from assert_type statements.

  assert_type is handled internally by pytype at type checking time; it should
  do nothing at runtime.
  """



"""Implementation of special members of third_party/six."""

from pytype import metaclass
from pytype.overlays import overlay


class SixOverlay(overlay.Overlay):
  """A custom overlay for the 'six' module."""

  def __init__(self, ctx):
    member_map = {
        "add_metaclass": build_add_metaclass,
        "with_metaclass": build_with_metaclass,
        "string_types": build_string_types,
        "integer_types": build_integer_types,
        "PY2": build_version_bool(2),
        "PY3": build_version_bool(3),
    }
    ast = ctx.loader.import_name("six")
    super().__init__(ctx, "six", member_map, ast)


def build_add_metaclass(ctx):
  return metaclass.AddMetaclass.make("add_metaclass", ctx, "six")


def build_with_metaclass(ctx):
  return metaclass.WithMetaclass.make("with_metaclass", ctx, "six")


def build_version_bool(major):
  return lambda ctx: ctx.convert.bool_values[ctx.python_version[0] == major]


def build_string_types(ctx):
  # six.string_types is defined as a tuple, even though it's only a single value
  # in Py3.
  # We're following the pyi definition of string_types here, because the real
  # value in Py2 is `basestring`, which we don't have available.
  classes = [ctx.convert.str_type.to_variable(ctx.root_node)]
  return ctx.convert.tuple_to_value(classes)


def build_integer_types(ctx):
  # pytype treats `long` as an alias of `int`, so the value of integer_types can
  # be represented as just `(int,)` in both Py2 and Py3.
  return ctx.convert.tuple_to_value(
      (ctx.convert.int_type.to_variable(ctx.root_node),))



from typing import Any

from sqlalchemy import sql, types as sqltypes
from sqlalchemy.engine import default
from sqlalchemy.sql import compiler
from sqlalchemy.types import (
    BIGINT as BIGINT,
    BLOB as BLOB,
    DATE as DATE,
    FLOAT as FLOAT,
    INTEGER as INTEGER,
    NUMERIC as NUMERIC,
    SMALLINT as SMALLINT,
    TEXT as TEXT,
    TIME as TIME,
    TIMESTAMP as TIMESTAMP,
    Integer as Integer,
)

RESERVED_WORDS: Any

class _StringType(sqltypes.String):
    charset: Any
    def __init__(self, charset: Any | None = ..., **kw) -> None: ...

class VARCHAR(_StringType, sqltypes.VARCHAR):
    __visit_name__: str
    def __init__(self, length: Any | None = ..., **kwargs) -> None: ...

class CHAR(_StringType, sqltypes.CHAR):
    __visit_name__: str
    def __init__(self, length: Any | None = ..., **kwargs) -> None: ...

class _FBDateTime(sqltypes.DateTime):
    def bind_processor(self, dialect): ...

colspecs: Any
ischema_names: Any

class FBTypeCompiler(compiler.GenericTypeCompiler):
    def visit_boolean(self, type_, **kw): ...
    def visit_datetime(self, type_, **kw): ...
    def visit_TEXT(self, type_, **kw): ...
    def visit_BLOB(self, type_, **kw): ...
    def visit_CHAR(self, type_, **kw): ...
    def visit_VARCHAR(self, type_, **kw): ...

class FBCompiler(sql.compiler.SQLCompiler):
    ansi_bind_rules: bool
    def visit_now_func(self, fn, **kw): ...
    def visit_startswith_op_binary(self, binary, operator, **kw): ...
    def visit_not_startswith_op_binary(self, binary, operator, **kw): ...
    def visit_mod_binary(self, binary, operator, **kw): ...
    def visit_alias(self, alias, asfrom: bool = ..., **kwargs): ...  # type: ignore[override]
    def visit_substring_func(self, func, **kw): ...
    def visit_length_func(self, function, **kw): ...
    visit_char_length_func: Any
    def function_argspec(self, func, **kw): ...
    def default_from(self): ...
    def visit_sequence(self, seq, **kw): ...
    def get_select_precolumns(self, select, **kw): ...
    def limit_clause(self, select, **kw): ...
    def returning_clause(self, stmt, returning_cols): ...

class FBDDLCompiler(sql.compiler.DDLCompiler):
    def visit_create_sequence(self, create): ...
    def visit_drop_sequence(self, drop): ...
    def visit_computed_column(self, generated): ...

class FBIdentifierPreparer(sql.compiler.IdentifierPreparer):
    reserved_words: Any
    illegal_initial_characters: Any
    def __init__(self, dialect) -> None: ...

class FBExecutionContext(default.DefaultExecutionContext):
    def fire_sequence(self, seq, type_): ...

class FBDialect(default.DefaultDialect):
    name: str
    supports_statement_cache: bool
    max_identifier_length: int
    supports_sequences: bool
    sequences_optional: bool
    supports_default_values: bool
    postfetch_lastrowid: bool
    supports_native_boolean: bool
    requires_name_normalize: bool
    supports_empty_insert: bool
    statement_compiler: Any
    ddl_compiler: Any
    preparer: Any
    type_compiler: Any
    colspecs: Any
    ischema_names: Any
    construct_arguments: Any
    def __init__(self, *args, **kwargs) -> None: ...
    implicit_returning: Any
    def initialize(self, connection) -> None: ...
    def has_table(self, connection, table_name, schema: Any | None = ...): ...  # type: ignore[override]
    def has_sequence(self, connection, sequence_name, schema: Any | None = ...): ...  # type: ignore[override]
    def get_table_names(self, connection, schema: Any | None = ..., **kw): ...
    def get_view_names(self, connection, schema: Any | None = ..., **kw): ...
    def get_view_definition(self, connection, view_name, schema: Any | None = ..., **kw): ...
    def get_pk_constraint(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_column_sequence(self, connection, table_name, column_name, schema: Any | None = ..., **kw): ...
    def get_columns(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_foreign_keys(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_indexes(self, connection, table_name, schema: Any | None = ..., **kw): ...



"""Class definitions in pyi files."""

import collections
import sys

from typing import Dict, List

from pytype.pyi.types import ParseError  # pylint: disable=g-importing-member
from pytype.pytd import pytd
from pytype.pytd import pytd_utils
from pytype.pytd.parse import node as pytd_node

# pylint: disable=g-import-not-at-top
if sys.version_info >= (3, 8):
  import ast as ast3
else:
  from typed_ast import ast3
# pylint: enable=g-import-not-at-top


_PROTOCOL_ALIASES = ("typing.Protocol", "typing_extensions.Protocol")


def get_bases(bases: List[pytd.Type]) -> List[pytd.Type]:
  """Collect base classes."""

  bases_out = []
  namedtuple_index = None
  for i, p in enumerate(bases):
    if p.name and pytd_utils.MatchesFullName(p, _PROTOCOL_ALIASES):
      bases_out.append(pytd.NamedType("typing.Protocol"))
      if isinstance(p, pytd.GenericType):
        # From PEP 544: "`Protocol[T, S, ...]` is allowed as a shorthand for
        # `Protocol, Generic[T, S, ...]`."
        # https://www.python.org/dev/peps/pep-0544/#generic-protocols
        bases_out.append(p.Replace(base_type=pytd.NamedType("typing.Generic")))
    elif isinstance(p, pytd.NamedType) and p.name == "typing.NamedTuple":
      if namedtuple_index is not None:
        raise ParseError("cannot inherit from bare NamedTuple more than once")
      namedtuple_index = i
      bases_out.append(p)
    elif isinstance(p, pytd.Type):
      bases_out.append(p)
    else:
      msg = f"Unexpected class base: {p}"
      raise ParseError(msg)
  return bases_out


def get_metaclass(keywords: List[ast3.keyword]):
  """Scan keywords for a metaclass."""

  for k in keywords:
    keyword, value = k.arg, k.value
    if keyword not in ("metaclass", "total"):
      raise ParseError(f"Unexpected classdef kwarg {keyword!r}")
    # TODO(rechen): We should store the "total" value instead of throwing it
    # away and validate in load_pytd that "total" is passed only to TypedDict
    # subclasses. We can't do the validation here because external types need to
    # be resolved first.
    if keyword == "metaclass":
      return value
  return None


def get_decorators(decorators: List[str], type_map: Dict[str, pytd_node.Node]):
  """Process a class decorator list."""

  # Drop the @type_check_only decorator from classes
  # TODO(mdemello): Workaround for the bug that typing.foo class decorators
  # don't add the import, since typing.type_check_only is the only one.
  decorators = [x for x in decorators if x != "type_check_only"]

  # Check for some function/method-only decorators
  nonclass = {"property", "classmethod", "staticmethod", "overload"}
  unsupported_decorators = set(decorators) & nonclass
  if unsupported_decorators:
    raise ParseError(
        f"Unsupported class decorators: {', '.join(unsupported_decorators)}")

  # Convert decorators to named types. These are wrapped as aliases because we
  # otherwise do not allow referencing functions as types.
  return [pytd.Alias(d, type_map.get(d) or pytd.NamedType(d))
          for d in decorators]


def check_for_duplicate_defs(methods, constants, aliases) -> None:
  """Check a class's list of definitions for duplicates."""
  all_names = (list({f.name for f in methods}) +
               [c.name for c in constants] +
               [a.name for a in aliases])
  duplicates = [name
                for name, count in collections.Counter(all_names).items()
                if count >= 2]
  if duplicates:
    raise ParseError(
        f"Duplicate class-level identifier(s): {', '.join(duplicates)}")



from typing import Any

TYPES_MAP: Any
REVERSE_TYPES_MAP: Any

def signature(*arguments): ...

class FunctionRegistry(type):
    def __init__(cls, name, bases, attrs) -> None: ...

class Functions:
    FUNCTION_TABLE: Any
    def call_function(self, function_name, resolved_args): ...



"""Implementation of TypedDict."""

import dataclasses

from typing import Any, Dict, Optional, Set

from pytype.abstract import abstract
from pytype.abstract import abstract_utils
from pytype.abstract import function
from pytype.overlays import classgen
from pytype.pytd import pytd


class TypedDictKeyMissing(function.DictKeyMissing):

  def __init__(self, typed_dict: "TypedDict", key: Optional[str]):
    super().__init__(key)
    self.typed_dict = typed_dict


@dataclasses.dataclass
class TypedDictProperties:
  """Collection of typed dict properties passed between various stages."""

  name: str
  fields: Dict[str, Any]
  required: Set[str]
  total: bool

  @property
  def keys(self):
    return set(self.fields.keys())

  @property
  def optional(self):
    return self.keys - self.required

  def add(self, k, v, total):
    self.fields[k] = v  # pylint: disable=unsupported-assignment-operation
    if total:
      self.required.add(k)

  def check_keys(self, keys):
    keys = set(keys)
    missing = (self.keys - keys) & self.required
    extra = keys - self.keys
    return missing, extra


class TypedDictBuilder(abstract.PyTDClass):
  """Factory for creating typing.TypedDict classes."""

  def __init__(self, ctx):
    typing_ast = ctx.loader.import_name("typing")
    pyval = typing_ast.Lookup("typing._TypedDict")
    pyval = pyval.Replace(name="typing.TypedDict")
    super().__init__("TypedDict", pyval, ctx)
    # Signature for the functional constructor
    fn = typing_ast.Lookup("typing._TypedDictFunction")
    fn = fn.Replace(name="typing.TypedDict")
    sig, = fn.signatures
    self.fn_sig = function.Signature.from_pytd(
        self.ctx, "typing.TypedDict", sig)

  def call(self, node, _, args):
    """Call the functional constructor."""
    props = self._extract_args(args)
    cls = TypedDictClass(props, self, self.ctx)
    cls_var = cls.to_variable(node)
    return node, cls_var

  def _extract_param(self, args, pos, name, pyval_type, typ):
    var = args.posargs[pos]
    try:
      return abstract_utils.get_atomic_python_constant(var, pyval_type)
    except abstract_utils.ConversionError as e:
      bad = function.BadParam(name, typ)
      raise function.WrongArgTypes(self.fn_sig, args, self.ctx, bad) from e

  def _extract_args(self, args):
    if len(args.posargs) != 2:
      raise function.WrongArgCount(self.fn_sig, args, self.ctx)
    name = self._extract_param(args, 0, "name", str, self.ctx.convert.str_type)
    fields = self._extract_param(
        args, 1, "fields", dict, self.ctx.convert.dict_type)
    props = TypedDictProperties(
        name=name, fields=fields, required=set(fields.keys()), total=True)
    return props

  def _validate_bases(self, cls_name, bases):
    """Check that all base classes are valid."""
    for base_var in bases:
      for base in base_var.data:
        if not isinstance(base, (TypedDictClass, TypedDictBuilder)):
          details = (f"TypedDict {cls_name} cannot inherit from "
                     "a non-TypedDict class.")
          self.ctx.errorlog.base_class_error(
              self.ctx.vm.frames, base_var, details)

  def _merge_base_class_fields(self, bases, props):
    """Add the merged list of base class fields to the fields dict."""
    # Updates props in place, raises an error if a duplicate key is encountered.
    provenance = {k: props.name for k in props.fields}
    for base_var in bases:
      for base in base_var.data:
        if not isinstance(base, TypedDictClass):
          continue
        for k, v in base.props.fields.items():
          if k in props.fields:
            classes = f"{base.name} and {provenance[k]}"
            details = f"Duplicate TypedDict key {k} in classes {classes}"
            self.ctx.errorlog.base_class_error(
                self.ctx.vm.frames, base_var, details)
          else:
            props.add(k, v, base.props.total)
            provenance[k] = base.name

  def make_class(self, node, bases, f_locals, total):
    # If BuildClass.call() hits max depth, f_locals will be [unsolvable]
    # See comment in NamedTupleClassBuilder.make_class(); equivalent logic
    # applies here.
    if isinstance(f_locals.data[0], abstract.Unsolvable):
      return node, self.ctx.new_unsolvable(node)

    f_locals = abstract_utils.get_atomic_python_constant(f_locals)

    # retrieve __qualname__ to get the name of class
    name_var = f_locals["__qualname__"]
    cls_name = abstract_utils.get_atomic_python_constant(name_var)
    if "." in cls_name:
      cls_name = cls_name.rsplit(".", 1)[-1]

    if total is None:
      total = True
    else:
      total = abstract_utils.get_atomic_python_constant(total, bool)
    props = TypedDictProperties(
        name=cls_name, fields={}, required=set(), total=total)

    # Collect the key types defined in the current class.
    cls_locals = classgen.get_class_locals(
        cls_name,
        allow_methods=False,
        ordering=classgen.Ordering.FIRST_ANNOTATE,
        ctx=self.ctx)
    for k, local in cls_locals.items():
      assert local.typ
      props.add(k, local.typ, total)

    # Process base classes and generate the __init__ signature.
    self._validate_bases(cls_name, bases)
    self._merge_base_class_fields(bases, props)

    cls = TypedDictClass(props, self, self.ctx)
    cls_var = cls.to_variable(node)
    return node, cls_var

  def make_class_from_pyi(self, cls_name, pytd_cls, total=True):
    """Make a TypedDictClass from a pyi class."""
    # NOTE: Returns the abstract class, not a variable.
    name = pytd_cls.name or cls_name
    if total is None:
      total = True
    props = TypedDictProperties(
        name=name, fields={}, required=set(), total=total)

    for c in pytd_cls.constants:
      typ = self.ctx.convert.constant_to_var(c.type)
      props.add(c.name, typ, total)

    # Process base classes and generate the __init__ signature.
    bases = [self.ctx.convert.constant_to_var(x)
             for x in pytd_cls.bases]
    self._validate_bases(cls_name, bases)
    self._merge_base_class_fields(bases, props)

    cls = TypedDictClass(props, self, self.ctx)
    return cls


class TypedDictClass(abstract.PyTDClass):
  """A template for typed dicts."""

  def __init__(self, props, base_cls, ctx):
    self.props = props
    self._base_cls = base_cls  # TypedDictBuilder for constructing subclasses
    super().__init__(props.name, ctx.convert.dict_type.pytd_cls, ctx)
    self.init_method = self._make_init(props)

  def __repr__(self):
    return f"TypedDictClass({self.name})"

  def _make_init(self, props):
    # __init__ method for type checking signatures.
    # We construct this here and pass it to TypedDictClass because we need
    # access to abstract.SignedFunction.
    sig = function.Signature.from_param_names(
        f"{props.name}.__init__", props.fields.keys(),
        kind=pytd.ParameterKind.KWONLY)
    sig.annotations = {k: abstract_utils.get_atomic_value(v)
                       for k, v in props.fields.items()}
    sig.defaults = {k: self.ctx.new_unsolvable(self.ctx.root_node)
                    for k in props.optional}
    return abstract.SignedFunction(sig, self.ctx)

  def _new_instance(self, container, node, args):
    self.init_method.match_and_map_args(node, args, {})
    ret = TypedDict(self.props, self.ctx)
    for (k, v) in args.namedargs.items():
      ret.set_str_item(node, k, v)
    return ret

  def instantiate_value(self, node, container):
    args = function.Args(())
    for name, typ in self.props.fields.items():
      args.namedargs[name] = self.ctx.join_variables(
          node, [t.instantiate(node) for t in typ.data])
    return self._new_instance(container, node, args)

  def instantiate(self, node, container=None):
    return self.instantiate_value(node, container).to_variable(node)

  def make_class(self, *args, **kwargs):
    return self._base_cls.make_class(*args, **kwargs)


class TypedDict(abstract.Dict):
  """Representation of TypedDict instances.

  Internally, a TypedDict is a dict with a restricted set of string keys
  allowed, each with a fixed type. We implement it as a subclass of Dict, with
  some type checks wrapped around key accesses. If a check fails, we simply add
  an error to the logs and then continue processing the method as though it were
  a regular dict.
  """

  def __init__(self, props, ctx):
    super().__init__(ctx)
    self.props = props
    self.set_slot("__delitem__", self.delitem_slot)

  @property
  def fields(self):
    return self.props.fields

  @property
  def class_name(self):
    return self.props.name

  def __repr__(self):
    return f"<TypedDict {self.class_name}>"

  def _check_str_key(self, name):
    if name not in self.fields:
      raise TypedDictKeyMissing(self, name)

  def _check_str_key_value(self, node, name, value_var):
    self._check_str_key(name)
    typ = abstract_utils.get_atomic_value(self.fields[name])
    bad, _ = self.ctx.matcher(node).bad_matches(value_var, typ)
    for view, error_details in bad:
      binding = view[value_var]
      self.ctx.errorlog.annotation_type_mismatch(
          self.ctx.vm.frames, typ, binding, name, error_details,
          typed_dict=self
      )

  def _check_key(self, name_var):
    """Check that key is in the typed dict."""
    try:
      name = abstract_utils.get_atomic_python_constant(name_var, str)
    except abstract_utils.ConversionError as e:
      raise TypedDictKeyMissing(self, None) from e
    self._check_str_key(name)

  def _check_value(self, node, name_var, value_var):
    """Check that value has the right type."""
    # We have already called check_key so name is in fields
    name = abstract_utils.get_atomic_python_constant(name_var, str)
    self._check_str_key_value(node, name, value_var)

  def getitem_slot(self, node, name_var):
    # A typed dict getitem should have a concrete string arg. If we have a var
    # with multiple bindings just fall back to Any.
    self._check_key(name_var)
    return super().getitem_slot(node, name_var)

  def setitem_slot(self, node, name_var, value_var):
    self._check_key(name_var)
    self._check_value(node, name_var, value_var)
    return super().setitem_slot(node, name_var, value_var)

  def set_str_item(self, node, name, value_var):
    self._check_str_key_value(node, name, value_var)
    return super().set_str_item(node, name, value_var)

  def delitem_slot(self, node, name_var):
    self._check_key(name_var)
    return self.call_pytd(node, "__delitem__", name_var)

  def pop_slot(self, node, key_var, default_var=None):
    self._check_key(key_var)
    return super().pop_slot(node, key_var, default_var)



from . import config as config, mock as mock
from .assertions import (
    AssertsCompiledSQL as AssertsCompiledSQL,
    AssertsExecutionResults as AssertsExecutionResults,
    ComparesTables as ComparesTables,
    assert_raises as assert_raises,
    assert_raises_context_ok as assert_raises_context_ok,
    assert_raises_message as assert_raises_message,
    assert_raises_message_context_ok as assert_raises_message_context_ok,
    emits_warning as emits_warning,
    emits_warning_on as emits_warning_on,
    eq_ as eq_,
    eq_ignore_whitespace as eq_ignore_whitespace,
    eq_regex as eq_regex,
    expect_deprecated as expect_deprecated,
    expect_deprecated_20 as expect_deprecated_20,
    expect_raises as expect_raises,
    expect_raises_message as expect_raises_message,
    expect_warnings as expect_warnings,
    in_ as in_,
    is_ as is_,
    is_false as is_false,
    is_instance_of as is_instance_of,
    is_none as is_none,
    is_not as is_not,
    is_not_ as is_not_,
    is_not_none as is_not_none,
    is_true as is_true,
    le_ as le_,
    ne_ as ne_,
    not_in as not_in,
    not_in_ as not_in_,
    startswith_ as startswith_,
    uses_deprecated as uses_deprecated,
)
from .config import (
    async_test as async_test,
    combinations as combinations,
    combinations_list as combinations_list,
    db as db,
    fixture as fixture,
)
from .exclusions import (
    db_spec as db_spec,
    exclude as exclude,
    fails as fails,
    fails_if as fails_if,
    fails_on as fails_on,
    fails_on_everything_except as fails_on_everything_except,
    future as future,
    only_if as only_if,
    only_on as only_on,
    skip as skip,
    skip_if as skip_if,
)
from .schema import eq_clause_element as eq_clause_element, eq_type_affinity as eq_type_affinity
from .util import (
    adict as adict,
    fail as fail,
    flag_combinations as flag_combinations,
    force_drop_names as force_drop_names,
    lambda_combinations as lambda_combinations,
    metadata_fixture as metadata_fixture,
    provide_metadata as provide_metadata,
    resolve_lambda as resolve_lambda,
    rowset as rowset,
    run_as_contextmanager as run_as_contextmanager,
    teardown_events as teardown_events,
)
from .warnings import assert_warnings as assert_warnings, warn_test_suite as warn_test_suite

def against(*queries): ...

crashes = skip



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class SourceContext(google.protobuf.message.Message):
    """`SourceContext` represents information about the source of a
    protobuf element, like the file in which it is defined.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    FILE_NAME_FIELD_NUMBER: builtins.int
    file_name: typing.Text
    """The path-qualified name of the .proto file that contained the associated
    protobuf element.  For example: `"google/protobuf/source_context.proto"`.
    """

    def __init__(self,
        *,
        file_name: typing.Optional[typing.Text] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["file_name",b"file_name"]) -> None: ...
global___SourceContext = SourceContext



from typing import Text

_EitherStr = bytes | Text

class PyCompileError(Exception):
    exc_type_name: str
    exc_value: BaseException
    file: str
    msg: str
    def __init__(self, exc_type: type[BaseException], exc_value: BaseException, file: str, msg: str = ...) -> None: ...

def compile(file: _EitherStr, cfile: _EitherStr | None = ..., dfile: _EitherStr | None = ..., doraise: bool = ...) -> None: ...
def main(args: list[Text] | None = ...) -> int: ...



from typing import Any, Pattern

from markdown.blockprocessors import BlockProcessor
from markdown.extensions import Extension
from markdown.inlinepatterns import InlineProcessor

ABBR_REF_RE: Pattern[str]

class AbbrExtension(Extension): ...
class AbbrPreprocessor(BlockProcessor): ...

class AbbrInlineProcessor(InlineProcessor):
    title: Any
    def __init__(self, pattern, title) -> None: ...

def makeExtension(**kwargs): ...



from _typeshed import Self
from abc import ABCMeta, abstractmethod
from typing import ClassVar

from cryptography.hazmat.backends.interfaces import EllipticCurveBackend
from cryptography.hazmat.primitives.asymmetric import AsymmetricVerificationContext
from cryptography.hazmat.primitives.asymmetric.utils import Prehashed
from cryptography.hazmat.primitives.hashes import HashAlgorithm
from cryptography.hazmat.primitives.serialization import Encoding, KeySerializationEncryption, PrivateFormat, PublicFormat
from cryptography.x509 import ObjectIdentifier

class EllipticCurve(metaclass=ABCMeta):
    @property
    @abstractmethod
    def key_size(self) -> int: ...
    @property
    @abstractmethod
    def name(self) -> str: ...

class BrainpoolP256R1(EllipticCurve):
    key_size: int = ...
    name: str = ...

class BrainpoolP384R1(EllipticCurve):
    key_size: int = ...
    name: str = ...

class BrainpoolP512R1(EllipticCurve):
    key_size: int = ...
    name: str = ...

class SECP192R1(EllipticCurve):
    key_size: int = ...
    name: str = ...

class SECP224R1(EllipticCurve):
    key_size: int = ...
    name: str = ...

class SECP256K1(EllipticCurve):
    key_size: int = ...
    name: str = ...

class SECP256R1(EllipticCurve):
    key_size: int = ...
    name: str = ...

class SECP384R1(EllipticCurve):
    key_size: int = ...
    name: str = ...

class SECP521R1(EllipticCurve):
    key_size: int = ...
    name: str = ...

class SECT163K1(EllipticCurve):
    key_size: int = ...
    name: str = ...

class SECT163R2(EllipticCurve):
    key_size: int = ...
    name: str = ...

class SECT233K1(EllipticCurve):
    key_size: int = ...
    name: str = ...

class SECT233R1(EllipticCurve):
    key_size: int = ...
    name: str = ...

class SECT283K1(EllipticCurve):
    key_size: int = ...
    name: str = ...

class SECT283R1(EllipticCurve):
    key_size: int = ...
    name: str = ...

class SECT409K1(EllipticCurve):
    key_size: int = ...
    name: str = ...

class SECT409R1(EllipticCurve):
    key_size: int = ...
    name: str = ...

class SECT571K1(EllipticCurve):
    key_size: int = ...
    name: str = ...

class SECT571R1(EllipticCurve):
    key_size: int = ...
    name: str = ...

class EllipticCurveOID:
    SECP192R1: ClassVar[ObjectIdentifier]
    SECP224R1: ClassVar[ObjectIdentifier]
    SECP256K1: ClassVar[ObjectIdentifier]
    SECP256R1: ClassVar[ObjectIdentifier]
    SECP384R1: ClassVar[ObjectIdentifier]
    SECP521R1: ClassVar[ObjectIdentifier]
    BRAINPOOLP256R1: ClassVar[ObjectIdentifier]
    BRAINPOOLP384R1: ClassVar[ObjectIdentifier]
    BRAINPOOLP512R1: ClassVar[ObjectIdentifier]
    SECT163K1: ClassVar[ObjectIdentifier]
    SECT163R2: ClassVar[ObjectIdentifier]
    SECT233K1: ClassVar[ObjectIdentifier]
    SECT233R1: ClassVar[ObjectIdentifier]
    SECT283K1: ClassVar[ObjectIdentifier]
    SECT283R1: ClassVar[ObjectIdentifier]
    SECT409K1: ClassVar[ObjectIdentifier]
    SECT409R1: ClassVar[ObjectIdentifier]
    SECT571K1: ClassVar[ObjectIdentifier]
    SECT571R1: ClassVar[ObjectIdentifier]

class EllipticCurvePrivateKey(metaclass=ABCMeta):
    @property
    @abstractmethod
    def curve(self) -> EllipticCurve: ...
    @property
    @abstractmethod
    def key_size(self) -> int: ...
    @abstractmethod
    def exchange(self, algorithm: ECDH, peer_public_key: EllipticCurvePublicKey) -> bytes: ...
    @abstractmethod
    def public_key(self) -> EllipticCurvePublicKey: ...
    @abstractmethod
    def sign(self, data: bytes, signature_algorithm: EllipticCurveSignatureAlgorithm) -> bytes: ...

class EllipticCurvePrivateKeyWithSerialization(EllipticCurvePrivateKey):
    @abstractmethod
    def private_bytes(
        self, encoding: Encoding, format: PrivateFormat, encryption_algorithm: KeySerializationEncryption
    ) -> bytes: ...
    @abstractmethod
    def private_numbers(self) -> EllipticCurvePrivateNumbers: ...

class EllipticCurvePrivateNumbers:
    @property
    def private_value(self) -> int: ...
    @property
    def public_numbers(self) -> EllipticCurvePublicNumbers: ...
    def __init__(self, private_value: int, public_numbers: EllipticCurvePublicNumbers) -> None: ...
    def private_key(self, backend: EllipticCurveBackend | None = ...) -> EllipticCurvePrivateKey: ...

class EllipticCurvePublicKey(metaclass=ABCMeta):
    @property
    @abstractmethod
    def curve(self) -> EllipticCurve: ...
    @property
    @abstractmethod
    def key_size(self) -> int: ...
    @classmethod
    def from_encoded_point(cls, curve: EllipticCurve, data: bytes) -> EllipticCurvePublicKey: ...
    @abstractmethod
    def public_bytes(self, encoding: Encoding, format: PublicFormat) -> bytes: ...
    @abstractmethod
    def public_numbers(self) -> EllipticCurvePublicNumbers: ...
    @abstractmethod
    def verifier(
        self, signature: bytes, signature_algorithm: EllipticCurveSignatureAlgorithm
    ) -> AsymmetricVerificationContext: ...
    @abstractmethod
    def verify(self, signature: bytes, data: bytes, signature_algorithm: EllipticCurveSignatureAlgorithm) -> None: ...

EllipticCurvePublicKeyWithSerialization = EllipticCurvePublicKey

class EllipticCurvePublicNumbers:
    @property
    def curve(self) -> EllipticCurve: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    def __init__(self, x: int, y: int, curve: EllipticCurve) -> None: ...
    @classmethod
    def from_encoded_point(cls: type[Self], curve: EllipticCurve, data: bytes) -> Self: ...
    def public_key(self, backend: EllipticCurveBackend | None = ...) -> EllipticCurvePublicKey: ...

class EllipticCurveSignatureAlgorithm(metaclass=ABCMeta):
    @property
    @abstractmethod
    def algorithm(self) -> HashAlgorithm | Prehashed: ...

class ECDH: ...

class ECDSA(EllipticCurveSignatureAlgorithm):
    def __init__(self, algorithm: HashAlgorithm | Prehashed): ...
    @property
    def algorithm(self) -> HashAlgorithm | Prehashed: ...

def derive_private_key(
    private_value: int, curve: EllipticCurve, backend: EllipticCurveBackend | None = ...
) -> EllipticCurvePrivateKey: ...
def generate_private_key(curve: EllipticCurve, backend: EllipticCurveBackend | None = ...) -> EllipticCurvePrivateKey: ...
def get_curve_for_oid(oid: ObjectIdentifier) -> EllipticCurve: ...



class ParserBase(object):
    def __init__(self) -> None: ...
    def error(self, message: str) -> None: ...
    def reset(self) -> None: ...
    def getpos(self) -> tuple[int, int]: ...
    def unknown_decl(self, data: str) -> None: ...



from typing import Sequence

cmp_op: Sequence[str]
hasconst: list[int]
hasname: list[int]
hasjrel: list[int]
hasjabs: list[int]
haslocal: list[int]
hascompare: list[int]
hasfree: list[int]
opname: list[str]

opmap: dict[str, int]
HAVE_ARGUMENT: int
EXTENDED_ARG: int



from typing import Any

from selenium.webdriver.remote.webdriver import WebDriver as RemoteWebDriver

LOAD_TIMEOUT: int

class WebDriver(RemoteWebDriver):
    def __init__(
        self, device_password, bb_tools_dir: Any | None = ..., hostip: str = ..., port: int = ..., desired_capabilities=...
    ): ...
    def quit(self) -> None: ...



import os
import sys
from _typeshed import Self
from types import TracebackType
from typing import IO, Any, AnyStr, Generic, Iterable, Iterator, overload
from typing_extensions import Literal

if sys.version_info >= (3, 9):
    from types import GenericAlias

__all__ = [
    "NamedTemporaryFile",
    "TemporaryFile",
    "SpooledTemporaryFile",
    "TemporaryDirectory",
    "mkstemp",
    "mkdtemp",
    "mktemp",
    "TMP_MAX",
    "gettempprefix",
    "tempdir",
    "gettempdir",
    "gettempprefixb",
    "gettempdirb",
]

# global variables
TMP_MAX: int
tempdir: str | None
template: str

_DirT = AnyStr | os.PathLike[AnyStr]

if sys.version_info >= (3, 8):
    @overload
    def NamedTemporaryFile(
        mode: Literal["r", "w", "a", "x", "r+", "w+", "a+", "x+", "rt", "wt", "at", "xt", "r+t", "w+t", "a+t", "x+t"],
        buffering: int = ...,
        encoding: str | None = ...,
        newline: str | None = ...,
        suffix: AnyStr | None = ...,
        prefix: AnyStr | None = ...,
        dir: _DirT[AnyStr] | None = ...,
        delete: bool = ...,
        *,
        errors: str | None = ...,
    ) -> _TemporaryFileWrapper[str]: ...
    @overload
    def NamedTemporaryFile(
        mode: Literal["rb", "wb", "ab", "xb", "r+b", "w+b", "a+b", "x+b"] = ...,
        buffering: int = ...,
        encoding: str | None = ...,
        newline: str | None = ...,
        suffix: AnyStr | None = ...,
        prefix: AnyStr | None = ...,
        dir: _DirT[AnyStr] | None = ...,
        delete: bool = ...,
        *,
        errors: str | None = ...,
    ) -> _TemporaryFileWrapper[bytes]: ...
    @overload
    def NamedTemporaryFile(
        mode: str = ...,
        buffering: int = ...,
        encoding: str | None = ...,
        newline: str | None = ...,
        suffix: AnyStr | None = ...,
        prefix: AnyStr | None = ...,
        dir: _DirT[AnyStr] | None = ...,
        delete: bool = ...,
        *,
        errors: str | None = ...,
    ) -> _TemporaryFileWrapper[Any]: ...

else:
    @overload
    def NamedTemporaryFile(
        mode: Literal["r", "w", "a", "x", "r+", "w+", "a+", "x+", "rt", "wt", "at", "xt", "r+t", "w+t", "a+t", "x+t"],
        buffering: int = ...,
        encoding: str | None = ...,
        newline: str | None = ...,
        suffix: AnyStr | None = ...,
        prefix: AnyStr | None = ...,
        dir: _DirT[AnyStr] | None = ...,
        delete: bool = ...,
    ) -> _TemporaryFileWrapper[str]: ...
    @overload
    def NamedTemporaryFile(
        mode: Literal["rb", "wb", "ab", "xb", "r+b", "w+b", "a+b", "x+b"] = ...,
        buffering: int = ...,
        encoding: str | None = ...,
        newline: str | None = ...,
        suffix: AnyStr | None = ...,
        prefix: AnyStr | None = ...,
        dir: _DirT[AnyStr] | None = ...,
        delete: bool = ...,
    ) -> _TemporaryFileWrapper[bytes]: ...
    @overload
    def NamedTemporaryFile(
        mode: str = ...,
        buffering: int = ...,
        encoding: str | None = ...,
        newline: str | None = ...,
        suffix: AnyStr | None = ...,
        prefix: AnyStr | None = ...,
        dir: _DirT[AnyStr] | None = ...,
        delete: bool = ...,
    ) -> _TemporaryFileWrapper[Any]: ...

if sys.platform == "win32":
    TemporaryFile = NamedTemporaryFile
else:
    if sys.version_info >= (3, 8):
        @overload
        def TemporaryFile(
            mode: Literal["r", "w", "a", "x", "r+", "w+", "a+", "x+", "rt", "wt", "at", "xt", "r+t", "w+t", "a+t", "x+t"],
            buffering: int = ...,
            encoding: str | None = ...,
            newline: str | None = ...,
            suffix: AnyStr | None = ...,
            prefix: AnyStr | None = ...,
            dir: _DirT[AnyStr] | None = ...,
            *,
            errors: str | None = ...,
        ) -> IO[str]: ...
        @overload
        def TemporaryFile(
            mode: Literal["rb", "wb", "ab", "xb", "r+b", "w+b", "a+b", "x+b"] = ...,
            buffering: int = ...,
            encoding: str | None = ...,
            newline: str | None = ...,
            suffix: AnyStr | None = ...,
            prefix: AnyStr | None = ...,
            dir: _DirT[AnyStr] | None = ...,
            *,
            errors: str | None = ...,
        ) -> IO[bytes]: ...
        @overload
        def TemporaryFile(
            mode: str = ...,
            buffering: int = ...,
            encoding: str | None = ...,
            newline: str | None = ...,
            suffix: AnyStr | None = ...,
            prefix: AnyStr | None = ...,
            dir: _DirT[AnyStr] | None = ...,
            *,
            errors: str | None = ...,
        ) -> IO[Any]: ...
    else:
        @overload
        def TemporaryFile(
            mode: Literal["r", "w", "a", "x", "r+", "w+", "a+", "x+", "rt", "wt", "at", "xt", "r+t", "w+t", "a+t", "x+t"],
            buffering: int = ...,
            encoding: str | None = ...,
            newline: str | None = ...,
            suffix: AnyStr | None = ...,
            prefix: AnyStr | None = ...,
            dir: _DirT[AnyStr] | None = ...,
        ) -> IO[str]: ...
        @overload
        def TemporaryFile(
            mode: Literal["rb", "wb", "ab", "xb", "r+b", "w+b", "a+b", "x+b"] = ...,
            buffering: int = ...,
            encoding: str | None = ...,
            newline: str | None = ...,
            suffix: AnyStr | None = ...,
            prefix: AnyStr | None = ...,
            dir: _DirT[AnyStr] | None = ...,
        ) -> IO[bytes]: ...
        @overload
        def TemporaryFile(
            mode: str = ...,
            buffering: int = ...,
            encoding: str | None = ...,
            newline: str | None = ...,
            suffix: AnyStr | None = ...,
            prefix: AnyStr | None = ...,
            dir: _DirT[AnyStr] | None = ...,
        ) -> IO[Any]: ...

class _TemporaryFileWrapper(Generic[AnyStr], IO[AnyStr]):
    file: IO[AnyStr]  # io.TextIOWrapper, io.BufferedReader or io.BufferedWriter
    name: str
    delete: bool
    def __init__(self, file: IO[AnyStr], name: str, delete: bool = ...) -> None: ...
    def __enter__(self: Self) -> Self: ...
    def __exit__(self, exc: type[BaseException] | None, value: BaseException | None, tb: TracebackType | None) -> None: ...
    def __getattr__(self, name: str) -> Any: ...
    def close(self) -> None: ...
    # These methods don't exist directly on this object, but
    # are delegated to the underlying IO object through __getattr__.
    # We need to add them here so that this class is concrete.
    def __iter__(self) -> Iterator[AnyStr]: ...
    # FIXME: __next__ doesn't actually exist on this class and should be removed:
    #        see also https://github.com/python/typeshed/pull/5456#discussion_r633068648
    # >>> import tempfile
    # >>> ntf=tempfile.NamedTemporaryFile()
    # >>> next(ntf)
    # Traceback (most recent call last):
    #   File "<stdin>", line 1, in <module>
    # TypeError: '_TemporaryFileWrapper' object is not an iterator
    def __next__(self) -> AnyStr: ...
    def fileno(self) -> int: ...
    def flush(self) -> None: ...
    def isatty(self) -> bool: ...
    def read(self, n: int = ...) -> AnyStr: ...
    def readable(self) -> bool: ...
    def readline(self, limit: int = ...) -> AnyStr: ...
    def readlines(self, hint: int = ...) -> list[AnyStr]: ...
    def seek(self, offset: int, whence: int = ...) -> int: ...
    def seekable(self) -> bool: ...
    def tell(self) -> int: ...
    def truncate(self, size: int | None = ...) -> int: ...
    def writable(self) -> bool: ...
    def write(self, s: AnyStr) -> int: ...
    def writelines(self, lines: Iterable[AnyStr]) -> None: ...

# It does not actually derive from IO[AnyStr], but it does implement the
# protocol.
class SpooledTemporaryFile(IO[AnyStr]):
    @property
    def encoding(self) -> str: ...  # undocumented
    @property
    def newlines(self) -> str | tuple[str, ...] | None: ...  # undocumented
    # bytes needs to go first, as default mode is to open as bytes
    if sys.version_info >= (3, 8):
        @overload
        def __init__(
            self: SpooledTemporaryFile[bytes],
            max_size: int = ...,
            mode: Literal["rb", "wb", "ab", "xb", "r+b", "w+b", "a+b", "x+b"] = ...,
            buffering: int = ...,
            encoding: str | None = ...,
            newline: str | None = ...,
            suffix: str | None = ...,
            prefix: str | None = ...,
            dir: str | None = ...,
            *,
            errors: str | None = ...,
        ) -> None: ...
        @overload
        def __init__(
            self: SpooledTemporaryFile[str],
            max_size: int = ...,
            mode: Literal["r", "w", "a", "x", "r+", "w+", "a+", "x+", "rt", "wt", "at", "xt", "r+t", "w+t", "a+t", "x+t"] = ...,
            buffering: int = ...,
            encoding: str | None = ...,
            newline: str | None = ...,
            suffix: str | None = ...,
            prefix: str | None = ...,
            dir: str | None = ...,
            *,
            errors: str | None = ...,
        ) -> None: ...
        @overload
        def __init__(
            self,
            max_size: int = ...,
            mode: str = ...,
            buffering: int = ...,
            encoding: str | None = ...,
            newline: str | None = ...,
            suffix: str | None = ...,
            prefix: str | None = ...,
            dir: str | None = ...,
            *,
            errors: str | None = ...,
        ) -> None: ...
        @property
        def errors(self) -> str | None: ...
    else:
        @overload
        def __init__(
            self: SpooledTemporaryFile[bytes],
            max_size: int = ...,
            mode: Literal["rb", "wb", "ab", "xb", "r+b", "w+b", "a+b", "x+b"] = ...,
            buffering: int = ...,
            encoding: str | None = ...,
            newline: str | None = ...,
            suffix: str | None = ...,
            prefix: str | None = ...,
            dir: str | None = ...,
        ) -> None: ...
        @overload
        def __init__(
            self: SpooledTemporaryFile[str],
            max_size: int = ...,
            mode: Literal["r", "w", "a", "x", "r+", "w+", "a+", "x+", "rt", "wt", "at", "xt", "r+t", "w+t", "a+t", "x+t"] = ...,
            buffering: int = ...,
            encoding: str | None = ...,
            newline: str | None = ...,
            suffix: str | None = ...,
            prefix: str | None = ...,
            dir: str | None = ...,
        ) -> None: ...
        @overload
        def __init__(
            self,
            max_size: int = ...,
            mode: str = ...,
            buffering: int = ...,
            encoding: str | None = ...,
            newline: str | None = ...,
            suffix: str | None = ...,
            prefix: str | None = ...,
            dir: str | None = ...,
        ) -> None: ...

    def rollover(self) -> None: ...
    def __enter__(self: Self) -> Self: ...
    def __exit__(self, exc: type[BaseException] | None, value: BaseException | None, tb: TracebackType | None) -> None: ...
    # These methods are copied from the abstract methods of IO, because
    # SpooledTemporaryFile implements IO.
    # See also https://github.com/python/typeshed/pull/2452#issuecomment-420657918.
    def close(self) -> None: ...
    def fileno(self) -> int: ...
    def flush(self) -> None: ...
    def isatty(self) -> bool: ...
    def read(self, n: int = ...) -> AnyStr: ...
    def readline(self, limit: int = ...) -> AnyStr: ...
    def readlines(self, hint: int = ...) -> list[AnyStr]: ...
    def seek(self, offset: int, whence: int = ...) -> int: ...
    def tell(self) -> int: ...
    def truncate(self, size: int | None = ...) -> None: ...  # type: ignore[override]
    def write(self, s: AnyStr) -> int: ...
    def writelines(self, iterable: Iterable[AnyStr]) -> None: ...
    def __iter__(self) -> Iterator[AnyStr]: ...
    # Other than the following methods, which do not exist on SpooledTemporaryFile
    def readable(self) -> bool: ...
    def seekable(self) -> bool: ...
    def writable(self) -> bool: ...
    def __next__(self) -> AnyStr: ...
    if sys.version_info >= (3, 9):
        def __class_getitem__(cls, item: Any) -> GenericAlias: ...

class TemporaryDirectory(Generic[AnyStr]):
    name: AnyStr
    if sys.version_info >= (3, 10):
        @overload
        def __init__(
            self: TemporaryDirectory[str],
            suffix: None = ...,
            prefix: None = ...,
            dir: None = ...,
            ignore_cleanup_errors: bool = ...,
        ) -> None: ...
        @overload
        def __init__(
            self,
            suffix: AnyStr | None = ...,
            prefix: AnyStr | None = ...,
            dir: _DirT[AnyStr] | None = ...,
            ignore_cleanup_errors: bool = ...,
        ) -> None: ...
    else:
        @overload
        def __init__(self: TemporaryDirectory[str], suffix: None = ..., prefix: None = ..., dir: None = ...) -> None: ...
        @overload
        def __init__(self, suffix: AnyStr | None = ..., prefix: AnyStr | None = ..., dir: _DirT[AnyStr] | None = ...) -> None: ...

    def cleanup(self) -> None: ...
    def __enter__(self) -> AnyStr: ...
    def __exit__(self, exc: type[BaseException] | None, value: BaseException | None, tb: TracebackType | None) -> None: ...
    if sys.version_info >= (3, 9):
        def __class_getitem__(cls, item: Any) -> GenericAlias: ...

@overload
def mkstemp() -> tuple[int, str]: ...
@overload
def mkstemp(
    suffix: AnyStr | None = ..., prefix: AnyStr | None = ..., dir: _DirT[AnyStr] | None = ..., text: bool = ...
) -> tuple[int, AnyStr]: ...
@overload
def mkdtemp() -> str: ...
@overload
def mkdtemp(suffix: AnyStr | None = ..., prefix: AnyStr | None = ..., dir: _DirT[AnyStr] | None = ...) -> AnyStr: ...
@overload
def mktemp() -> str: ...
@overload
def mktemp(suffix: AnyStr | None = ..., prefix: AnyStr | None = ..., dir: _DirT[AnyStr] | None = ...) -> AnyStr: ...
def gettempdirb() -> bytes: ...
def gettempprefixb() -> bytes: ...
def gettempdir() -> str: ...
def gettempprefix() -> str: ...



"""Converts pyi files to pickled asts and saves them to disk.

Used to speed up module importing. This is done by loading the ast and
serializing it to disk. Further users only need to read the serialized data from
disk, which is faster to digest than a pyi file.
"""

from typing import List, NamedTuple, Optional, Set, Tuple

from pytype import utils
from pytype.pyi import parser
from pytype.pytd import pytd
from pytype.pytd import pytd_utils
from pytype.pytd import visitors


class UnrestorableDependencyError(Exception):
  """If a dependency can't be restored in the current state."""


class FindClassTypesVisitor(visitors.Visitor):
  """Visitor to find class and function types."""

  def __init__(self):
    super().__init__()
    self.class_type_nodes = []

  def EnterClassType(self, n):
    self.class_type_nodes.append(n)


class UndoModuleAliasesVisitor(visitors.Visitor):
  """Visitor to undo module aliases in late types.

  Since late types are loaded out of context, they need to contain the original
  names of modules, not whatever they've been aliased to in the current module.
  """

  def __init__(self):
    super().__init__()
    self._module_aliases = {}

  def EnterTypeDeclUnit(self, node):
    for alias in node.aliases:
      if isinstance(alias.type, pytd.Module):
        name = utils.strip_prefix(alias.name, f"{node.name}.")
        self._module_aliases[name] = alias.type.module_name

  def VisitLateType(self, node):
    if "." not in node.name:
      return node
    prefix, suffix = node.name.rsplit(".", 1)
    while prefix:
      if prefix in self._module_aliases:
        return node.Replace(name=self._module_aliases[prefix] + "." + suffix)
      prefix, _, remainder = prefix.rpartition(".")
      suffix = f"{remainder}.{suffix}"
    return node


class SerializableTupleClass(NamedTuple):
  ast: pytd.TypeDeclUnit
  dependencies: List[Tuple[str, Set[str]]]
  late_dependencies: List[Tuple[str, Set[str]]]
  class_type_nodes: Optional[List[pytd.ClassType]]
  is_package: bool  # TODO(rechen): remove this, can be computed from src_path
  src_path: Optional[str]
  metadata: List[str]


class SerializableAst(SerializableTupleClass):
  """The data pickled to disk to save an ast.

  Attributes:
    ast: The TypeDeclUnit representing the serialized module.
    dependencies: A list of modules this AST depends on. The modules are
      represented as Fully Qualified names. E.g. foo.bar.module. This set will
      also contain the module being imported, if the module is not empty.
      Therefore it might be different from the set found by
      visitors.CollectDependencies in
      load_pytd._load_and_resolve_ast_dependencies.
    late_dependencies: This AST's late dependencies.
    class_type_nodes: A list of all the ClassType instances in ast or None. If
      this list is provided only the ClassType instances in the list will be
      visited and have their .cls set. If this attribute is None the whole AST
      will be visited and all found ClassType instances will have their .cls
      set.
    is_package: True if the original source file was a package __init__.
    src_path: Optionally, the filepath of the original source file.
    metadata: A list of arbitrary string-encoded metadata.
  """
  Replace = SerializableTupleClass._replace  # pylint: disable=no-member,invalid-name


def StoreAst(ast, filename=None, open_function=open, is_package=False,
             src_path=None, metadata=None):
  """Loads and stores an ast to disk.

  Args:
    ast: The pytd.TypeDeclUnit to save to disk.
    filename: The filename for the pickled output. If this is None, this
      function instead returns the pickled string.
    open_function: A custom file opening function.
    is_package: Whether the module with the given ast is a package.
    src_path: Optionally, the filepath of the original source file.
    metadata: A list of arbitrary string-encoded metadata.

  Returns:
    The pickled string, if no filename was given. (None otherwise.)
  """
  if ast.name.endswith(".__init__"):
    assert is_package
    ast = ast.Visit(visitors.RenameModuleVisitor(
        ast.name, ast.name.rsplit(".__init__", 1)[0]))
  ast = ast.Visit(UndoModuleAliasesVisitor())
  # Collect dependencies
  deps = visitors.CollectDependencies()
  ast.Visit(deps)
  dependencies = deps.dependencies
  late_dependencies = deps.late_dependencies

  # Clean external references
  ast.Visit(visitors.ClearClassPointers())
  indexer = FindClassTypesVisitor()
  ast.Visit(indexer)
  ast = ast.Visit(visitors.CanonicalOrderingVisitor())

  metadata = metadata or []

  return pytd_utils.SavePickle(
      SerializableAst(
          ast, sorted(dependencies.items()), sorted(late_dependencies.items()),
          sorted(indexer.class_type_nodes), is_package=is_package,
          src_path=src_path, metadata=metadata,
      ),
      filename, open_function=open_function)


def EnsureAstName(ast, module_name, fix=False):
  """Verify that serializable_ast has the name module_name, or repair it.

  Args:
    ast: An instance of SerializableAst.
    module_name: The name under which ast.ast should be loaded.
    fix: If this function should repair the wrong name.

  Returns:
    The updated SerializableAst.
  """
  # The most likely case is module_name==raw_ast.name .
  raw_ast = ast.ast

  # module_name is the name from this run, raw_ast.name is the guessed name from
  # when the ast has been pickled.
  if fix and module_name != raw_ast.name:
    ast = ast.Replace(class_type_nodes=None)
    ast = ast.Replace(ast=raw_ast.Visit(
        visitors.RenameModuleVisitor(raw_ast.name, module_name)))
  else:
    assert module_name == raw_ast.name
  return ast


def ProcessAst(serializable_ast, module_map):
  """Postprocess a pickled ast.

  Postprocessing will either just fill the ClassType references from module_map
  or if module_name changed between pickling and loading rename the module
  internal references to the new module_name.
  Renaming is more expensive than filling references, as the whole AST needs to
  be rebuild.

  Args:
    serializable_ast: A SerializableAst instance.
    module_map: Used to resolve ClassType.cls links to already loaded modules.
      The loaded module will be added to the dict.

  Returns:
    A pytd.TypeDeclUnit, this is either the input raw_ast with the references
    set or a newly created AST with the new module_name and the references set.

  Raises:
    AssertionError: If module_name is already in module_map, which means that
      module_name is already loaded.
    UnrestorableDependencyError: If no concrete module exists in module_map for
      one of the references from the pickled ast.
  """
  # Module external and internal references need to be filled in different
  # steps. As a part of a local ClassType referencing an external cls, might be
  # changed structurally, if the external class definition used here is
  # different from the one used during serialization. Changing an attribute
  # (other than .cls) will trigger an recreation of the ClassType in which case
  # we need the reference to the new instance, which can only be known after all
  # external references are resolved.
  serializable_ast = _LookupClassReferences(
      serializable_ast, module_map, serializable_ast.ast.name)
  serializable_ast = FillLocalReferences(serializable_ast, {
      "": serializable_ast.ast,
      serializable_ast.ast.name: serializable_ast.ast})
  return serializable_ast.ast


def _LookupClassReferences(serializable_ast, module_map, self_name):
  """Fills .cls references in serializable_ast.ast with ones from module_map.

  Already filled references are not changed. References to the module self._name
  are not filled. Setting self_name=None will fill all references.

  Args:
    serializable_ast: A SerializableAst instance.
    module_map: Used to resolve ClassType.cls links to already loaded modules.
      The loaded module will be added to the dict.
    self_name: A string representation of a module which should not be resolved,
      for example: "foo.bar.module1" or None to resolve all modules.

  Returns:
    A SerializableAst with an updated .ast. .class_type_nodes is set to None
    if any of the Nodes needed to be regenerated.
  """

  class_lookup = visitors.LookupExternalTypes(module_map, self_name=self_name)
  raw_ast = serializable_ast.ast

  decorators = {d.type.name for c in raw_ast.classes for d in c.decorators}  # pylint: disable=g-complex-comprehension

  for node in (serializable_ast.class_type_nodes or ()):
    try:
      class_lookup.allow_functions = node.name in decorators
      if node is not class_lookup.VisitClassType(node):
        serializable_ast = serializable_ast.Replace(class_type_nodes=None)
        break
    except KeyError as e:
      raise UnrestorableDependencyError(
          f"Unresolved class: {utils.message(e)!r}.") from e
  if serializable_ast.class_type_nodes is None:
    try:
      raw_ast = raw_ast.Visit(class_lookup)
    except KeyError as e:
      raise UnrestorableDependencyError(
          f"Unresolved class: {utils.message(e)!r}.") from e
  serializable_ast = serializable_ast.Replace(ast=raw_ast)
  return serializable_ast


def FillLocalReferences(serializable_ast, module_map):
  """Fill in local references."""
  local_filler = visitors.FillInLocalPointers(module_map)
  if serializable_ast.class_type_nodes is None:
    serializable_ast.ast.Visit(local_filler)
    return serializable_ast.Replace(class_type_nodes=None)
  else:
    for node in serializable_ast.class_type_nodes:
      local_filler.EnterClassType(node)
      if node.cls is None:
        raise AssertionError(f"This should not happen: {str(node)}")
    return serializable_ast


def PrepareForExport(module_name, ast, loader):
  """Prepare an ast as if it was parsed and loaded.

  External dependencies will not be resolved, as the ast generated by this
  method is supposed to be exported.

  Args:
    module_name: The module_name as a string for the returned ast.
    ast: pytd.TypeDeclUnit, is only used if src is None.
    loader: A load_pytd.Loader instance.

  Returns:
    A pytd.TypeDeclUnit representing the supplied AST as it would look after
    being written to a file and parsed.
  """
  # This is a workaround for functionality which crept into places it doesn't
  # belong. Ideally this would call some transformation Visitors on ast to
  # transform it into the same ast we get after parsing and loading (compare
  # load_pytd.Loader.load_file). Unfortunately parsing has some special cases,
  # e.g. '__init__' return type and '__new__' being a 'staticmethod', which
  # need to be moved to visitors before we can do this. Printing an ast also
  # applies transformations,
  # e.g. visitors.PrintVisitor._FormatContainerContents, which need to move to
  # their own visitors so they can be applied without printing.
  src = pytd_utils.Print(ast)
  return SourceToExportableAst(module_name, src, loader)


def SourceToExportableAst(module_name, src, loader):
  """Parse the source code into a pickle-able ast."""
  ast = parser.parse_string(
      src=src, name=module_name,
      options=parser.PyiOptions.from_toplevel_options(loader.options))
  ast = ast.Visit(visitors.LookupBuiltins(loader.builtins, full_names=False))
  ast = ast.Visit(visitors.ExpandCompatibleBuiltins(loader.builtins))
  ast = ast.Visit(visitors.LookupLocalTypes())
  ast = ast.Visit(visitors.AdjustTypeParameters())
  ast = ast.Visit(visitors.NamedTypeToClassType())
  ast = ast.Visit(visitors.FillInLocalPointers({"": ast, module_name: ast}))
  ast = ast.Visit(visitors.ClassTypeToLateType(
      ignore=[module_name + ".", "builtins.", "typing."]))
  return ast



from _typeshed import SupportsKeysAndGetItem
from typing import Any, Generator, Iterable, Iterator, Mapping, MutableMapping, Sized

class URIDict(MutableMapping[str, str]):
    def normalize(self, uri: str) -> str: ...
    store: dict[str, str]
    def __init__(self, __m: SupportsKeysAndGetItem[str, str] | Iterable[tuple[str, str]], **kwargs: str) -> None: ...
    def __getitem__(self, uri: str) -> str: ...
    def __setitem__(self, uri: str, value: str) -> None: ...
    def __delitem__(self, uri: str) -> None: ...
    def __iter__(self) -> Iterator[str]: ...
    def __len__(self) -> int: ...

class Unset: ...

def load_schema(name): ...
def format_as_index(container: str, indices) -> str: ...
def find_additional_properties(instance: Iterable[Any], schema: Mapping[Any, Any]) -> Generator[Any, None, None]: ...
def extras_msg(extras: Iterable[Any] | Sized) -> str: ...
def ensure_list(thing) -> list[Any]: ...
def equal(one, two) -> bool: ...
def unbool(element, true=..., false=...): ...
def uniq(container) -> bool: ...
def find_evaluated_item_indexes_by_schema(validator, instance, schema) -> list[Any]: ...
def find_evaluated_property_keys_by_schema(validator, instance, schema) -> list[Any]: ...



"""Implementation of special members of typing_extensions."""

from pytype.overlays import typing_overlay


class TypingExtensionsOverlay(typing_overlay.Redirect):
  """A custom overlay for the 'typing_extensions' module."""

  def __init__(self, ctx):
    aliases = {"runtime": "typing.runtime_checkable"}
    super().__init__("typing_extensions", aliases, ctx)

  def _convert_member(self, name, member, subst=None):
    var = super()._convert_member(name, member, subst)
    for val in var.data:
      # typing_extensions backports typing features to older versions.
      # Pretending that the backports are in typing is easier than remembering
      # to check for both typing.X and typing_extensions.X every time we match
      # on an abstract value.
      val.module = "typing"
    return var



from collections.abc import Iterator, Mapping, Set as AbstractSet
from typing_extensions import TypedDict

from pygments.token import _TokenType

ansicolors: AbstractSet[str]  # not intended to be mutable

class _StyleDict(TypedDict):
    color: str | None
    bold: bool
    italic: bool
    underline: bool
    bgcolor: str | None
    border: str | None
    roman: bool | None  # lol yes, can be True or False or None
    sans: bool | None
    mono: bool | None
    ansicolor: str | None
    bgansicolor: str | None

class StyleMeta(type):
    def __new__(cls, name, bases, dct): ...
    def style_for_token(cls, token: _TokenType) -> _StyleDict: ...
    def styles_token(cls, ttype: _TokenType) -> bool: ...
    def list_styles(cls) -> list[tuple[_TokenType, _StyleDict]]: ...
    def __iter__(cls) -> Iterator[tuple[_TokenType, _StyleDict]]: ...
    def __len__(cls) -> int: ...
    # These are a bit tricky.
    # Technically should be ClassVar in class Style.
    # But then we can't use StyleMeta to denote a style class.
    # We need that because Type[Style] is not iterable, for example.
    background_color: str
    highlight_color: str
    line_number_color: str
    line_number_background_color: str
    line_number_special_color: str
    line_number_special_background_color: str
    styles: Mapping[_TokenType, str]  # not intended to be mutable

class Style(metaclass=StyleMeta): ...



from typing import Any

from ..util import memoized_property
from .base import Pool

class QueuePool(Pool):
    def __init__(
        self, creator, pool_size: int = ..., max_overflow: int = ..., timeout: float = ..., use_lifo: bool = ..., **kw
    ) -> None: ...
    def recreate(self): ...
    def dispose(self) -> None: ...
    def status(self): ...
    def size(self): ...
    def timeout(self): ...
    def checkedin(self): ...
    def overflow(self): ...
    def checkedout(self): ...

class AsyncAdaptedQueuePool(QueuePool): ...
class FallbackAsyncAdaptedQueuePool(AsyncAdaptedQueuePool): ...

class NullPool(Pool):
    def status(self): ...
    def recreate(self): ...
    def dispose(self) -> None: ...

class SingletonThreadPool(Pool):
    size: Any
    def __init__(self, creator, pool_size: int = ..., **kw) -> None: ...
    def recreate(self): ...
    def dispose(self) -> None: ...
    def status(self): ...
    def connect(self): ...

class StaticPool(Pool):
    @memoized_property
    def connection(self): ...
    def status(self): ...
    def dispose(self) -> None: ...
    def recreate(self): ...

class AssertionPool(Pool):
    def __init__(self, *args, **kw) -> None: ...
    def status(self): ...
    def dispose(self) -> None: ...
    def recreate(self): ...



from _typeshed import SupportsRead, SupportsReadline
from socket import socket
from ssl import SSLContext
from typing import Any, BinaryIO, Callable, Text
from typing_extensions import Literal

_IntOrStr = int | Text

MSG_OOB: int
FTP_PORT: int
MAXLINE: int
CRLF: str

class Error(Exception): ...
class error_reply(Error): ...
class error_temp(Error): ...
class error_perm(Error): ...
class error_proto(Error): ...

all_errors: tuple[type[Exception], ...]

class FTP:
    debugging: int

    # Note: This is technically the type that's passed in as the host argument.  But to make it easier in Python 2 we
    # accept Text but return str.
    host: str

    port: int
    maxline: int
    sock: socket | None
    welcome: str | None
    passiveserver: int
    timeout: int
    af: int
    lastresp: str

    file: BinaryIO | None
    def __init__(
        self, host: Text = ..., user: Text = ..., passwd: Text = ..., acct: Text = ..., timeout: float = ...
    ) -> None: ...
    def connect(self, host: Text = ..., port: int = ..., timeout: float = ...) -> str: ...
    def getwelcome(self) -> str: ...
    def set_debuglevel(self, level: int) -> None: ...
    def debug(self, level: int) -> None: ...
    def set_pasv(self, val: bool | int) -> None: ...
    def sanitize(self, s: Text) -> str: ...
    def putline(self, line: Text) -> None: ...
    def putcmd(self, line: Text) -> None: ...
    def getline(self) -> str: ...
    def getmultiline(self) -> str: ...
    def getresp(self) -> str: ...
    def voidresp(self) -> str: ...
    def abort(self) -> str: ...
    def sendcmd(self, cmd: Text) -> str: ...
    def voidcmd(self, cmd: Text) -> str: ...
    def sendport(self, host: Text, port: int) -> str: ...
    def sendeprt(self, host: Text, port: int) -> str: ...
    def makeport(self) -> socket: ...
    def makepasv(self) -> tuple[str, int]: ...
    def login(self, user: Text = ..., passwd: Text = ..., acct: Text = ...) -> str: ...
    # In practice, `rest` rest can actually be anything whose str() is an integer sequence, so to make it simple we allow integers.
    def ntransfercmd(self, cmd: Text, rest: _IntOrStr | None = ...) -> tuple[socket, int]: ...
    def transfercmd(self, cmd: Text, rest: _IntOrStr | None = ...) -> socket: ...
    def retrbinary(
        self, cmd: Text, callback: Callable[[bytes], Any], blocksize: int = ..., rest: _IntOrStr | None = ...
    ) -> str: ...
    def storbinary(
        self,
        cmd: Text,
        fp: SupportsRead[bytes],
        blocksize: int = ...,
        callback: Callable[[bytes], Any] | None = ...,
        rest: _IntOrStr | None = ...,
    ) -> str: ...
    def retrlines(self, cmd: Text, callback: Callable[[str], Any] | None = ...) -> str: ...
    def storlines(self, cmd: Text, fp: SupportsReadline[bytes], callback: Callable[[bytes], Any] | None = ...) -> str: ...
    def acct(self, password: Text) -> str: ...
    def nlst(self, *args: Text) -> list[str]: ...
    # Technically only the last arg can be a Callable but ...
    def dir(self, *args: str | Callable[[str], None]) -> None: ...
    def rename(self, fromname: Text, toname: Text) -> str: ...
    def delete(self, filename: Text) -> str: ...
    def cwd(self, dirname: Text) -> str: ...
    def size(self, filename: Text) -> int | None: ...
    def mkd(self, dirname: Text) -> str: ...
    def rmd(self, dirname: Text) -> str: ...
    def pwd(self) -> str: ...
    def quit(self) -> str: ...
    def close(self) -> None: ...

class FTP_TLS(FTP):
    def __init__(
        self,
        host: Text = ...,
        user: Text = ...,
        passwd: Text = ...,
        acct: Text = ...,
        keyfile: str | None = ...,
        certfile: str | None = ...,
        context: SSLContext | None = ...,
        timeout: float = ...,
        source_address: tuple[str, int] | None = ...,
    ) -> None: ...
    ssl_version: int
    keyfile: str | None
    certfile: str | None
    context: SSLContext
    def login(self, user: Text = ..., passwd: Text = ..., acct: Text = ..., secure: bool = ...) -> str: ...
    def auth(self) -> str: ...
    def prot_p(self) -> str: ...
    def prot_c(self) -> str: ...

class Netrc:
    def __init__(self, filename: Text | None = ...) -> None: ...
    def get_hosts(self) -> list[str]: ...
    def get_account(self, host: Text) -> tuple[str | None, str | None, str | None]: ...
    def get_macros(self) -> list[str]: ...
    def get_macro(self, macro: Text) -> tuple[str, ...]: ...

def parse150(resp: str) -> int | None: ...  # undocumented
def parse227(resp: str) -> tuple[str, int]: ...  # undocumented
def parse229(resp: str, peer: Any) -> tuple[str, int]: ...  # undocumented
def parse257(resp: str) -> str: ...  # undocumented
def ftpcp(
    source: FTP, sourcename: str, target: FTP, targetname: str = ..., type: Literal["A", "I"] = ...
) -> None: ...  # undocumented



from typing import Any

class Keys:
    NULL: str
    CANCEL: str
    HELP: str
    BACKSPACE: str
    BACK_SPACE: Any
    TAB: str
    CLEAR: str
    RETURN: str
    ENTER: str
    SHIFT: str
    LEFT_SHIFT: Any
    CONTROL: str
    LEFT_CONTROL: Any
    ALT: str
    LEFT_ALT: Any
    PAUSE: str
    ESCAPE: str
    SPACE: str
    PAGE_UP: str
    PAGE_DOWN: str
    END: str
    HOME: str
    LEFT: str
    ARROW_LEFT: Any
    UP: str
    ARROW_UP: Any
    RIGHT: str
    ARROW_RIGHT: Any
    DOWN: str
    ARROW_DOWN: Any
    INSERT: str
    DELETE: str
    SEMICOLON: str
    EQUALS: str
    NUMPAD0: str
    NUMPAD1: str
    NUMPAD2: str
    NUMPAD3: str
    NUMPAD4: str
    NUMPAD5: str
    NUMPAD6: str
    NUMPAD7: str
    NUMPAD8: str
    NUMPAD9: str
    MULTIPLY: str
    ADD: str
    SEPARATOR: str
    SUBTRACT: str
    DECIMAL: str
    DIVIDE: str
    F1: str
    F2: str
    F3: str
    F4: str
    F5: str
    F6: str
    F7: str
    F8: str
    F9: str
    F10: str
    F11: str
    F12: str
    META: str
    COMMAND: str



from _typeshed import Self
from typing import ClassVar

import passlib.utils.handlers as uh

class phpass(uh.HasManyIdents, uh.HasRounds, uh.HasSalt, uh.GenericHandler):  # type: ignore[misc]
    name: ClassVar[str]
    checksum_chars: ClassVar[str]
    min_salt_size: ClassVar[int]
    max_salt_size: ClassVar[int]
    salt_chars: ClassVar[str]
    default_rounds: ClassVar[int]
    min_rounds: ClassVar[int]
    max_rounds: ClassVar[int]
    rounds_cost: ClassVar[str]
    default_ident: ClassVar[str]
    ident_values: ClassVar[tuple[str, ...]]
    ident_aliases: ClassVar[dict[str, str]]
    @classmethod
    def from_string(cls: type[Self], hash: str | bytes) -> Self: ...  # type: ignore[override]



class SQLRole:
    allows_lambda: bool
    uses_inspection: bool

class UsesInspection:
    uses_inspection: bool

class AllowsLambdaRole:
    allows_lambda: bool

class HasCacheKeyRole(SQLRole): ...
class ExecutableOptionRole(SQLRole): ...
class LiteralValueRole(SQLRole): ...
class ColumnArgumentRole(SQLRole): ...
class ColumnArgumentOrKeyRole(ColumnArgumentRole): ...
class StrAsPlainColumnRole(ColumnArgumentRole): ...
class ColumnListRole(SQLRole): ...
class TruncatedLabelRole(SQLRole): ...
class ColumnsClauseRole(AllowsLambdaRole, UsesInspection, ColumnListRole): ...
class LimitOffsetRole(SQLRole): ...
class ByOfRole(ColumnListRole): ...
class GroupByRole(AllowsLambdaRole, UsesInspection, ByOfRole): ...
class OrderByRole(AllowsLambdaRole, ByOfRole): ...
class StructuralRole(SQLRole): ...
class StatementOptionRole(StructuralRole): ...
class OnClauseRole(AllowsLambdaRole, StructuralRole): ...
class WhereHavingRole(OnClauseRole): ...
class ExpressionElementRole(SQLRole): ...
class ConstExprRole(ExpressionElementRole): ...
class LabeledColumnExprRole(ExpressionElementRole): ...
class BinaryElementRole(ExpressionElementRole): ...
class InElementRole(SQLRole): ...
class JoinTargetRole(AllowsLambdaRole, UsesInspection, StructuralRole): ...
class FromClauseRole(ColumnsClauseRole, JoinTargetRole): ...

class StrictFromClauseRole(FromClauseRole):
    @property
    def description(self) -> None: ...

class AnonymizedFromClauseRole(StrictFromClauseRole): ...
class ReturnsRowsRole(SQLRole): ...
class StatementRole(SQLRole): ...

class SelectStatementRole(StatementRole, ReturnsRowsRole):
    def subquery(self) -> None: ...

class HasCTERole(ReturnsRowsRole): ...
class IsCTERole(SQLRole): ...
class CompoundElementRole(AllowsLambdaRole, SQLRole): ...
class DMLRole(StatementRole): ...
class DMLTableRole(FromClauseRole): ...
class DMLColumnRole(SQLRole): ...
class DMLSelectRole(SQLRole): ...
class DDLRole(StatementRole): ...
class DDLExpressionRole(StructuralRole): ...
class DDLConstraintColumnRole(SQLRole): ...
class DDLReferredColumnRole(DDLConstraintColumnRole): ...



from typing import IO, Any, Callable, Iterator, Union

HIGHEST_PROTOCOL: int
bytes_types: tuple[type[Any], ...]  # undocumented

def dump(obj: Any, file: IO[bytes], protocol: int | None = ...) -> None: ...
def dumps(obj: Any, protocol: int | None = ...) -> bytes: ...
def load(file: IO[bytes]) -> Any: ...
def loads(string: bytes) -> Any: ...

class PickleError(Exception): ...
class PicklingError(PickleError): ...
class UnpicklingError(PickleError): ...

_reducedtype = Union[
    str,
    tuple[Callable[..., Any], tuple[Any, ...]],
    tuple[Callable[..., Any], tuple[Any, ...], Any],
    tuple[Callable[..., Any], tuple[Any, ...], Any, Iterator[Any] | None],
    tuple[Callable[..., Any], tuple[Any, ...], Any, Iterator[Any] | None, Iterator[Any] | None],
]

class Pickler:
    fast: bool
    def __init__(self, file: IO[bytes], protocol: int | None = ...) -> None: ...
    def dump(self, __obj: Any) -> None: ...
    def clear_memo(self) -> None: ...
    def persistent_id(self, obj: Any) -> Any: ...

class Unpickler:
    def __init__(self, file: IO[bytes]) -> None: ...
    def load(self) -> Any: ...
    def find_class(self, __module_name: str, __global_name: str) -> Any: ...

MARK: bytes
STOP: bytes
POP: bytes
POP_MARK: bytes
DUP: bytes
FLOAT: bytes
INT: bytes
BININT: bytes
BININT1: bytes
LONG: bytes
BININT2: bytes
NONE: bytes
PERSID: bytes
BINPERSID: bytes
REDUCE: bytes
STRING: bytes
BINSTRING: bytes
SHORT_BINSTRING: bytes
UNICODE: bytes
BINUNICODE: bytes
APPEND: bytes
BUILD: bytes
GLOBAL: bytes
DICT: bytes
EMPTY_DICT: bytes
APPENDS: bytes
GET: bytes
BINGET: bytes
INST: bytes
LONG_BINGET: bytes
LIST: bytes
EMPTY_LIST: bytes
OBJ: bytes
PUT: bytes
BINPUT: bytes
LONG_BINPUT: bytes
SETITEM: bytes
TUPLE: bytes
EMPTY_TUPLE: bytes
SETITEMS: bytes
BINFLOAT: bytes

TRUE: bytes
FALSE: bytes

# protocol 2
PROTO: bytes
NEWOBJ: bytes
EXT1: bytes
EXT2: bytes
EXT4: bytes
TUPLE1: bytes
TUPLE2: bytes
TUPLE3: bytes
NEWTRUE: bytes
NEWFALSE: bytes
LONG1: bytes
LONG4: bytes

def encode_long(x: int) -> bytes: ...  # undocumented
def decode_long(data: bytes) -> int: ...  # undocumented



from abc import abstractmethod
from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

from ._chart import ChartBase

class _PieChartBase(ChartBase):
    varyColors: Any
    ser: Any
    dLbls: Any
    dataLabels: Any
    __elements__: Any
    def __init__(self, varyColors: bool = ..., ser=..., dLbls: Any | None = ...) -> None: ...
    @property
    @abstractmethod
    def tagname(self) -> str: ...

class PieChart(_PieChartBase):
    tagname: str
    varyColors: Any
    ser: Any
    dLbls: Any
    firstSliceAng: Any
    extLst: Any
    __elements__: Any
    def __init__(self, firstSliceAng: int = ..., extLst: Any | None = ..., **kw) -> None: ...

class PieChart3D(_PieChartBase):
    tagname: str
    varyColors: Any
    ser: Any
    dLbls: Any
    extLst: Any
    __elements__: Any

class DoughnutChart(_PieChartBase):
    tagname: str
    varyColors: Any
    ser: Any
    dLbls: Any
    firstSliceAng: Any
    holeSize: Any
    extLst: Any
    __elements__: Any
    def __init__(self, firstSliceAng: int = ..., holeSize: int = ..., extLst: Any | None = ..., **kw) -> None: ...

class CustomSplit(Serialisable):
    tagname: str
    secondPiePt: Any
    __elements__: Any
    def __init__(self, secondPiePt=...) -> None: ...

class ProjectedPieChart(_PieChartBase):
    tagname: str
    varyColors: Any
    ser: Any
    dLbls: Any
    ofPieType: Any
    type: Any
    gapWidth: Any
    splitType: Any
    splitPos: Any
    custSplit: Any
    secondPieSize: Any
    serLines: Any
    join_lines: Any
    extLst: Any
    __elements__: Any
    def __init__(
        self,
        ofPieType: str = ...,
        gapWidth: Any | None = ...,
        splitType: str = ...,
        splitPos: Any | None = ...,
        custSplit: Any | None = ...,
        secondPieSize: int = ...,
        serLines: Any | None = ...,
        extLst: Any | None = ...,
        **kw,
    ) -> None: ...



__all__ = (
    "CancelledError",
    "InvalidStateError",
    "TimeoutError",
    "IncompleteReadError",
    "LimitOverrunError",
    "SendfileNotAvailableError",
)

class CancelledError(BaseException): ...
class TimeoutError(Exception): ...
class InvalidStateError(Exception): ...
class SendfileNotAvailableError(RuntimeError): ...

class IncompleteReadError(EOFError):
    expected: int | None
    partial: bytes
    def __init__(self, partial: bytes, expected: int | None) -> None: ...

class LimitOverrunError(Exception):
    consumed: int
    def __init__(self, message: str, consumed: int) -> None: ...



from typing import Any

def get_active_sheet(wb): ...

class WorkbookWriter:
    wb: Any
    rels: Any
    package: Any
    def __init__(self, wb) -> None: ...
    def write_properties(self) -> None: ...
    def write_worksheets(self) -> None: ...
    def write_refs(self) -> None: ...
    def write_names(self) -> None: ...
    def write_pivots(self) -> None: ...
    def write_views(self) -> None: ...
    def write(self): ...
    def write_rels(self): ...
    def write_root_rels(self): ...



from typing import Any, Callable

from google.protobuf.descriptor import Descriptor, FieldDescriptor
from google.protobuf.message import Message

_Decoder = Callable[[str, int, int, Message, dict[FieldDescriptor, Any]], int]
_NewDefault = Callable[[Message], Message]

def ReadTag(buffer, pos): ...

Int32Decoder: _Decoder
Int64Decoder: _Decoder
UInt32Decoder: _Decoder
UInt64Decoder: _Decoder
SInt32Decoder: _Decoder
SInt64Decoder: _Decoder
Fixed32Decoder: _Decoder
Fixed64Decoder: _Decoder
SFixed32Decoder: _Decoder
SFixed64Decoder: _Decoder
FloatDecoder: _Decoder
DoubleDecoder: _Decoder
BoolDecoder: _Decoder

def EnumDecoder(
    field_number: int,
    is_repeated: bool,
    is_packed: bool,
    key: FieldDescriptor,
    new_default: _NewDefault,
    clear_if_default: bool = ...,
) -> _Decoder: ...
def StringDecoder(
    field_number: int,
    is_repeated: bool,
    is_packed: bool,
    key: FieldDescriptor,
    new_default: _NewDefault,
    clear_if_default: bool = ...,
) -> _Decoder: ...
def BytesDecoder(
    field_number: int,
    is_repeated: bool,
    is_packed: bool,
    key: FieldDescriptor,
    new_default: _NewDefault,
    clear_if_default: bool = ...,
) -> _Decoder: ...
def GroupDecoder(
    field_number: int, is_repeated: bool, is_packed: bool, key: FieldDescriptor, new_default: _NewDefault
) -> _Decoder: ...
def MessageDecoder(
    field_number: int, is_repeated: bool, is_packed: bool, key: FieldDescriptor, new_default: _NewDefault
) -> _Decoder: ...

MESSAGE_SET_ITEM_TAG: bytes

def MessageSetItemDecoder(descriptor: Descriptor) -> _Decoder: ...
def MapDecoder(field_descriptor, new_default, is_message_map) -> _Decoder: ...

SkipField: Any



from typing import Any

PATTERN: Any

class FreshnessDateDataParser:
    def get_local_tz(self): ...
    def parse(self, date_string, settings): ...
    def get_kwargs(self, date_string): ...
    def get_date_data(self, date_string, settings: Any | None = ...): ...

freshness_date_parser: Any



from typing import Any

from .base import BaseEndpoint as BaseEndpoint

log: Any

class RevocationEndpoint(BaseEndpoint):
    valid_token_types: Any
    valid_request_methods: Any
    request_validator: Any
    supported_token_types: Any
    enable_jsonp: Any
    def __init__(self, request_validator, supported_token_types: Any | None = ..., enable_jsonp: bool = ...) -> None: ...
    def create_revocation_response(self, uri, http_method: str = ..., body: Any | None = ..., headers: Any | None = ...): ...
    def validate_revocation_request(self, request) -> None: ...



import builtins
from typing import Any

import MySQLdb._exceptions

version_info: tuple[Any, ...]

class DataError(MySQLdb._exceptions.DatabaseError): ...
class DatabaseError(MySQLdb._exceptions.Error): ...
class Error(MySQLdb._exceptions.MySQLError): ...
class IntegrityError(MySQLdb._exceptions.DatabaseError): ...
class InterfaceError(MySQLdb._exceptions.Error): ...
class InternalError(MySQLdb._exceptions.DatabaseError): ...
class MySQLError(Exception): ...
class NotSupportedError(MySQLdb._exceptions.DatabaseError): ...
class OperationalError(MySQLdb._exceptions.DatabaseError): ...
class ProgrammingError(MySQLdb._exceptions.DatabaseError): ...
class Warning(builtins.Warning, MySQLdb._exceptions.MySQLError): ...

class connection:
    client_flag: Any
    converter: Any
    open: Any
    port: Any
    server_capabilities: Any
    def __init__(self, *args, **kwargs) -> None: ...
    def _get_native_connection(self, *args, **kwargs) -> Any: ...
    def affected_rows(self, *args, **kwargs) -> Any: ...
    def autocommit(self, on) -> Any: ...
    def change_user(self, *args, **kwargs) -> Any: ...
    def character_set_name(self, *args, **kwargs) -> Any: ...
    def close(self, *args, **kwargs) -> Any: ...
    def commit(self, *args, **kwargs) -> Any: ...
    def dump_debug_info(self, *args, **kwargs) -> Any: ...
    def errno(self, *args, **kwargs) -> Any: ...
    def error(self, *args, **kwargs) -> Any: ...
    def escape(self, obj, dict) -> Any: ...
    def escape_string(self, s) -> Any: ...
    def field_count(self, *args, **kwargs) -> Any: ...
    def fileno(self, *args, **kwargs) -> Any: ...
    def get_autocommit(self, *args, **kwargs) -> Any: ...
    def get_character_set_info(self, *args, **kwargs) -> Any: ...
    def get_host_info(self, *args, **kwargs) -> Any: ...
    def get_proto_info(self, *args, **kwargs) -> Any: ...
    def get_server_info(self, *args, **kwargs) -> Any: ...
    def info(self, *args, **kwargs) -> Any: ...
    def insert_id(self, *args, **kwargs) -> Any: ...
    def kill(self, *args, **kwargs) -> Any: ...
    def next_result(self) -> Any: ...
    def ping(self) -> Any: ...
    def query(self, query) -> Any: ...
    def read_query_result(self, *args, **kwargs) -> Any: ...
    def rollback(self, *args, **kwargs) -> Any: ...
    def select_db(self, *args, **kwargs) -> Any: ...
    def send_query(self, *args, **kwargs) -> Any: ...
    def set_character_set(self, charset) -> Any: ...
    def set_server_option(self, option) -> Any: ...
    def shutdown(self, *args, **kwargs) -> Any: ...
    def sqlstate(self, *args, **kwargs) -> Any: ...
    def stat(self, *args, **kwargs) -> Any: ...
    def store_result(self, *args, **kwargs) -> Any: ...
    def string_literal(self, obj) -> Any: ...
    def thread_id(self, *args, **kwargs) -> Any: ...
    def use_result(self, *args, **kwargs) -> Any: ...
    def warning_count(self, *args, **kwargs) -> Any: ...
    def __delattr__(self, __name) -> Any: ...
    def __setattr__(self, __name, __value) -> Any: ...

class result:
    converter: Any
    has_next: Any
    def __init__(self, *args, **kwargs) -> None: ...
    def data_seek(self, n) -> Any: ...
    def describe(self, *args, **kwargs) -> Any: ...
    def fetch_row(self, *args, **kwargs) -> Any: ...
    def field_flags(self, *args, **kwargs) -> Any: ...
    def num_fields(self, *args, **kwargs) -> Any: ...
    def num_rows(self, *args, **kwargs) -> Any: ...
    def __delattr__(self, __name) -> Any: ...
    def __setattr__(self, __name, __value) -> Any: ...

def connect(*args, **kwargs) -> Any: ...
def debug(*args, **kwargs) -> Any: ...
def escape(obj, dict) -> Any: ...
def escape_string(s) -> Any: ...
def get_client_info() -> Any: ...
def string_literal(obj) -> Any: ...



from _typeshed import Self, StrOrBytesPath
from typing import Any, Callable, TypeVar
from typing_extensions import ParamSpec

__all__ = ["run", "runctx", "Profile"]

def run(statement: str, filename: str | None = ..., sort: str | int = ...) -> None: ...
def runctx(
    statement: str, globals: dict[str, Any], locals: dict[str, Any], filename: str | None = ..., sort: str | int = ...
) -> None: ...

_T = TypeVar("_T")
_P = ParamSpec("_P")
_Label = tuple[str, int, str]

class Profile:
    bias: int
    stats: dict[_Label, tuple[int, int, int, int, dict[_Label, tuple[int, int, int, int]]]]  # undocumented
    def __init__(self, timer: Callable[[], float] | None = ..., bias: int | None = ...) -> None: ...
    def set_cmd(self, cmd: str) -> None: ...
    def simulate_call(self, name: str) -> None: ...
    def simulate_cmd_complete(self) -> None: ...
    def print_stats(self, sort: str | int = ...) -> None: ...
    def dump_stats(self, file: StrOrBytesPath) -> None: ...
    def create_stats(self) -> None: ...
    def snapshot_stats(self) -> None: ...
    def run(self: Self, cmd: str) -> Self: ...
    def runctx(self: Self, cmd: str, globals: dict[str, Any], locals: dict[str, Any]) -> Self: ...
    def runcall(self, __func: Callable[_P, _T], *args: _P.args, **kw: _P.kwargs) -> _T: ...
    def calibrate(self, m: int, verbose: int = ...) -> float: ...



import sys
from _typeshed import ReadableBuffer, Self
from typing import Iterable, Iterator, NoReturn, Sized, overload

ACCESS_DEFAULT: int
ACCESS_READ: int
ACCESS_WRITE: int
ACCESS_COPY: int

ALLOCATIONGRANULARITY: int

if sys.platform == "linux":
    MAP_DENYWRITE: int
    MAP_EXECUTABLE: int
    if sys.version_info >= (3, 10):
        MAP_POPULATE: int

if sys.platform != "win32":
    MAP_ANON: int
    MAP_ANONYMOUS: int
    MAP_PRIVATE: int
    MAP_SHARED: int
    PROT_EXEC: int
    PROT_READ: int
    PROT_WRITE: int

    PAGESIZE: int

class mmap(Iterable[int], Sized):
    if sys.platform == "win32":
        def __init__(self, fileno: int, length: int, tagname: str | None = ..., access: int = ..., offset: int = ...) -> None: ...
    else:
        def __init__(
            self, fileno: int, length: int, flags: int = ..., prot: int = ..., access: int = ..., offset: int = ...
        ) -> None: ...

    def close(self) -> None: ...
    if sys.version_info >= (3, 8):
        def flush(self, offset: int = ..., size: int = ...) -> None: ...
    else:
        def flush(self, offset: int = ..., size: int = ...) -> int: ...

    def move(self, dest: int, src: int, count: int) -> None: ...
    def read_byte(self) -> int: ...
    def readline(self) -> bytes: ...
    def resize(self, newsize: int) -> None: ...
    def seek(self, pos: int, whence: int = ...) -> None: ...
    def size(self) -> int: ...
    def tell(self) -> int: ...
    def write_byte(self, byte: int) -> None: ...
    def __len__(self) -> int: ...
    closed: bool
    if sys.version_info >= (3, 8) and sys.platform != "win32":
        def madvise(self, option: int, start: int = ..., length: int = ...) -> None: ...

    def find(self, sub: ReadableBuffer, start: int = ..., stop: int = ...) -> int: ...
    def rfind(self, sub: ReadableBuffer, start: int = ..., stop: int = ...) -> int: ...
    def read(self, n: int | None = ...) -> bytes: ...
    def write(self, bytes: ReadableBuffer) -> int: ...
    @overload
    def __getitem__(self, __index: int) -> int: ...
    @overload
    def __getitem__(self, __index: slice) -> bytes: ...
    def __delitem__(self, __index: int | slice) -> NoReturn: ...
    @overload
    def __setitem__(self, __index: int, __object: int) -> None: ...
    @overload
    def __setitem__(self, __index: slice, __object: ReadableBuffer) -> None: ...
    # Doesn't actually exist, but the object is actually iterable because it has __getitem__ and
    # __len__, so we claim that there is also an __iter__ to help type checkers.
    def __iter__(self) -> Iterator[int]: ...
    def __enter__(self: Self) -> Self: ...
    def __exit__(self, *args: object) -> None: ...

if sys.version_info >= (3, 8) and sys.platform != "win32":
    MADV_NORMAL: int
    MADV_RANDOM: int
    MADV_SEQUENTIAL: int
    MADV_WILLNEED: int
    MADV_DONTNEED: int

    if sys.platform == "linux":
        MADV_REMOVE: int
        MADV_DONTFORK: int
        MADV_DOFORK: int
        MADV_HWPOISON: int
        MADV_MERGEABLE: int
        MADV_UNMERGEABLE: int
        # Seems like this constant is not defined in glibc.
        # See https://github.com/python/typeshed/pull/5360 for details
        # MADV_SOFT_OFFLINE: int
        MADV_HUGEPAGE: int
        MADV_NOHUGEPAGE: int
        MADV_DONTDUMP: int
        MADV_DODUMP: int
        MADV_FREE: int

    # This Values are defined for FreeBSD but type checkers do not support conditions for these
    if sys.platform != "linux" and sys.platform != "darwin":
        MADV_NOSYNC: int
        MADV_AUTOSYNC: int
        MADV_NOCORE: int
        MADV_CORE: int
        MADV_PROTECT: int

if sys.version_info >= (3, 10) and sys.platform == "darwin":
    MADV_FREE_REUSABLE: int
    MADV_FREE_REUSE: int



from typing import Any

from braintree.error_result import ErrorResult as ErrorResult
from braintree.successful_result import SuccessfulResult as SuccessfulResult
from braintree.transaction import Transaction as Transaction

class TestingGateway:
    gateway: Any
    config: Any
    def __init__(self, gateway) -> None: ...
    def make_past_due(self, subscription_id, number_of_days_past_due: int = ...) -> None: ...
    def escrow_transaction(self, transaction_id) -> None: ...
    def settle_transaction(self, transaction_id): ...
    def settlement_confirm_transaction(self, transaction_id): ...
    def settlement_decline_transaction(self, transaction_id): ...
    def settlement_pending_transaction(self, transaction_id): ...
    def create_3ds_verification(self, merchant_account_id, params): ...



from concurrent.futures import Future
from typing import Callable, Text

from google.protobuf.descriptor import MethodDescriptor, ServiceDescriptor
from google.protobuf.message import Message

class RpcException(Exception): ...

class Service:
    @staticmethod
    def GetDescriptor() -> ServiceDescriptor: ...
    def CallMethod(
        self,
        method_descriptor: MethodDescriptor,
        rpc_controller: RpcController,
        request: Message,
        done: Callable[[Message], None] | None,
    ) -> Future[Message] | None: ...
    def GetRequestClass(self, method_descriptor: MethodDescriptor) -> type[Message]: ...
    def GetResponseClass(self, method_descriptor: MethodDescriptor) -> type[Message]: ...

class RpcController:
    def Reset(self) -> None: ...
    def Failed(self) -> bool: ...
    def ErrorText(self) -> Text | None: ...
    def StartCancel(self) -> None: ...
    def SetFailed(self, reason: Text) -> None: ...
    def IsCanceled(self) -> bool: ...
    def NotifyOnCancel(self, callback: Callable[[], None]) -> None: ...

class RpcChannel:
    def CallMethod(
        self,
        method_descriptor: MethodDescriptor,
        rpc_controller: RpcController,
        request: Message,
        response_class: type[Message],
        done: Callable[[Message], None] | None,
    ) -> Future[Message] | None: ...



from distutils.command.build_ext import build_ext as _build_ext
from typing import Any

have_rtld: bool
use_stubs: bool
libtype: str

def if_dl(s): ...
def get_abi3_suffix(): ...

class build_ext(_build_ext):
    inplace: Any
    def run(self) -> None: ...
    def copy_extensions_to_source(self) -> None: ...
    def get_ext_filename(self, fullname): ...
    shlib_compiler: Any
    shlibs: Any
    ext_map: Any
    def initialize_options(self) -> None: ...
    extensions: Any
    def finalize_options(self) -> None: ...
    def setup_shlib_compiler(self) -> None: ...
    def get_export_symbols(self, ext): ...
    compiler: Any
    def build_extension(self, ext) -> None: ...
    def links_to_dynamic(self, ext): ...
    def get_outputs(self): ...
    def write_stub(self, output_dir, ext, compile: bool = ...) -> None: ...

def link_shared_object(
    self,
    objects,
    output_libname,
    output_dir: Any | None = ...,
    libraries: Any | None = ...,
    library_dirs: Any | None = ...,
    runtime_library_dirs: Any | None = ...,
    export_symbols: Any | None = ...,
    debug: int = ...,
    extra_preargs: Any | None = ...,
    extra_postargs: Any | None = ...,
    build_temp: Any | None = ...,
    target_lang: Any | None = ...,
) -> None: ...



import io
from _typeshed import Self
from http.client import HTTPMessage as _HttplibHTTPMessage, HTTPResponse as _HttplibHTTPResponse
from typing import Any, Iterator, Mapping
from typing_extensions import Literal

from urllib3.connectionpool import HTTPConnection

from . import HTTPConnectionPool, Retry
from ._collections import HTTPHeaderDict
from .connection import _TYPE_BODY

class DeflateDecoder:
    def __init__(self) -> None: ...
    def __getattr__(self, name: str) -> Any: ...
    def decompress(self, data: bytes) -> bytes: ...

class GzipDecoderState:
    FIRST_MEMBER: Literal[0]
    OTHER_MEMBERS: Literal[1]
    SWALLOW_DATA: Literal[2]

class GzipDecoder:
    def __init__(self) -> None: ...
    def __getattr__(self, name: str) -> Any: ...
    def decompress(self, data: bytes) -> bytes: ...

# This class is only available if
# `brotli` is available for import.
class BrotliDecoder:
    def __init__(self) -> None: ...
    def flush(self) -> bytes: ...

class MultiDecoder:
    def __init__(self, modes: str) -> None: ...
    def flush(self) -> bytes: ...
    def decompress(self, data: bytes) -> bytes: ...

class HTTPResponse(io.IOBase):
    CONTENT_DECODERS: list[str]
    REDIRECT_STATUSES: list[int]
    headers: HTTPHeaderDict
    status: int
    version: int
    reason: str | None
    strict: int
    decode_content: bool
    retries: Retry | None
    enforce_content_length: bool
    auto_close: bool
    msg: _HttplibHTTPMessage | None
    chunked: bool
    chunk_left: int | None
    length_remaining: int | None
    def __init__(
        self,
        body: _TYPE_BODY = ...,
        headers: Mapping[str, str] | Mapping[bytes, bytes] | None = ...,
        status: int = ...,
        version: int = ...,
        reason: str | None = ...,
        strict: int = ...,
        preload_content: bool = ...,
        decode_content: bool = ...,
        original_response: _HttplibHTTPResponse | None = ...,
        pool: HTTPConnectionPool | None = ...,
        connection: HTTPConnection | None = ...,
        msg: _HttplibHTTPMessage | None = ...,
        retries: Retry | None = ...,
        enforce_content_length: bool = ...,
        request_method: str | None = ...,
        request_url: str | None = ...,
        auto_close: bool = ...,
    ) -> None: ...
    def get_redirect_location(self) -> Literal[False] | str | None: ...
    def release_conn(self) -> None: ...
    def drain_conn(self) -> None: ...
    @property
    def data(self) -> bytes | Any: ...
    @property
    def connection(self) -> HTTPConnection | Any: ...
    def isclosed(self) -> bool: ...
    def tell(self) -> int: ...
    def read(self, amt: int | None = ..., decode_content: bool | None = ..., cache_content: bool = ...) -> bytes: ...
    def stream(self, amt: int | None = ..., decode_content: bool | None = ...) -> Iterator[bytes]: ...
    @classmethod
    def from_httplib(cls: type[Self], r: _HttplibHTTPResponse, **response_kw: Any) -> Self: ...
    def getheaders(self) -> HTTPHeaderDict: ...
    def getheader(self, name, default=...) -> str | None: ...
    def info(self) -> HTTPHeaderDict: ...
    def close(self) -> None: ...
    @property
    def closed(self) -> bool: ...
    def fileno(self) -> int: ...
    def flush(self) -> None: ...
    def readable(self) -> bool: ...
    def readinto(self, b: bytearray) -> int: ...
    def supports_chunked_reads(self) -> bool: ...
    def read_chunked(self, amt: int | None = ..., decode_content: bool | None = ...) -> Iterator[bytes]: ...
    def geturl(self) -> bool | str: ...



from contextlib import AbstractContextManager
from typing import Any, TypeVar, overload
from typing_extensions import Literal

from .client import Pipeline, Redis, _StrType

_T = TypeVar("_T")

HIREDIS_AVAILABLE: bool
CRYPTOGRAPHY_AVAILABLE: bool

@overload
def from_url(url: str, *, db: int = ..., decode_responses: Literal[True], **kwargs: Any) -> Redis[str]: ...
@overload
def from_url(url: str, *, db: int = ..., decode_responses: Literal[False] = ..., **kwargs: Any) -> Redis[bytes]: ...
def pipeline(redis_obj: Redis[_StrType]) -> AbstractContextManager[Pipeline[_StrType]]: ...
@overload
def str_if_bytes(value: bytes) -> str: ...  # type: ignore[misc]
@overload
def str_if_bytes(value: _T) -> _T: ...
def safe_str(value: object) -> str: ...
def dict_merge(*dicts): ...
def list_keys_to_dict(key_list, callback): ...
def merge_result(command, res): ...



"""Solver for type equations."""

import itertools
import logging

from pytype.pytd import booleq
from pytype.pytd import escape
from pytype.pytd import optimize
from pytype.pytd import pytd
from pytype.pytd import pytd_utils
from pytype.pytd import transforms
from pytype.pytd import type_match
from pytype.pytd import visitors

log = logging.getLogger(__name__)

# How deep to nest type parameters
# TODO(b/159041279): Currently, the solver only generates variables for depth 1.
MAX_DEPTH = 1

is_unknown = type_match.is_unknown
is_partial = escape.is_partial
is_complete = escape.is_complete


class FlawedQuery(Exception):  # pylint: disable=g-bad-exception-name
  """Thrown if there is a fundamental flaw in the query."""


class TypeSolver:
  """Class for solving ~unknowns in type inference results."""

  def __init__(self, ast, builtins, protocols):
    self.ast = ast
    self.builtins = builtins
    self.protocols = protocols

  def match_unknown_against_protocol(self, matcher,
                                     solver, unknown, complete):
    """Given an ~unknown, match it against a class.

    Args:
      matcher: An instance of pytd.type_match.TypeMatch.
      solver: An instance of pytd.booleq.Solver.
      unknown: The unknown class to match
      complete: A complete class to match against. (E.g. a built-in or a user
        defined class)
    Returns:
      An instance of pytd.booleq.BooleanTerm.
    """

    assert is_unknown(unknown)
    assert is_complete(complete)
    type_params = {p.type_param: matcher.type_parameter(unknown, complete, p)
                   for p in complete.template}
    subst = type_params.copy()
    implication = matcher.match_Protocol_against_Unknown(
        complete, unknown, subst)
    if implication is not booleq.FALSE and type_params:
      # If we're matching against a templated class (E.g. list[T]), record the
      # fact that we'll also have to solve the type parameters.
      for param in type_params.values():
        solver.register_variable(param.name)
    solver.implies(booleq.Eq(unknown.name, complete.name), implication)

  def match_partial_against_complete(self, matcher, solver, partial, complete):
    """Match a partial class (call record) against a complete class.

    Args:
      matcher: An instance of pytd.type_match.TypeMatch.
      solver: An instance of pytd.booleq.Solver.
      partial: The partial class to match. The class name needs to be prefixed
        with "~" - the rest of the name is typically the same as complete.name.
      complete: A complete class to match against. (E.g. a built-in or a user
        defined class)
    Returns:
      An instance of pytd.booleq.BooleanTerm.
    Raises:
      FlawedQuery: If this call record is incompatible with the builtin.
    """
    assert is_partial(partial)
    assert is_complete(complete)
    # Types recorded for type parameters in the partial builtin are meaningless,
    # since we don't know which instance of the builtin used them when.
    subst = {p.type_param: pytd.AnythingType() for p in complete.template}
    formula = matcher.match_Class_against_Class(partial, complete, subst)
    if formula is booleq.FALSE:
      raise FlawedQuery(f"{partial.name} can never be {complete.name}")
    solver.always_true(formula)

  def match_call_record(self, matcher, solver, call_record, complete):
    """Match the record of a method call against the formal signature."""
    assert is_partial(call_record)
    assert is_complete(complete)
    formula = (
        matcher.match_Function_against_Function(call_record, complete, {}))
    if formula is booleq.FALSE:
      cartesian = call_record.Visit(visitors.ExpandSignatures())
      for signature in cartesian.signatures:
        formula = matcher.match_Signature_against_Function(
            signature, complete, {})
        if formula is booleq.FALSE:
          faulty_signature = pytd_utils.Print(signature)
          break
      else:
        faulty_signature = ""
      raise FlawedQuery("Bad call\n{}{}\nagainst:\n{}".format(
          escape.unpack_partial(call_record.name),
          faulty_signature, pytd_utils.Print(complete)))
    solver.always_true(formula)

  def solve(self):
    """Solve the equations generated from the pytd.

    Returns:
      A dictionary (str->str), mapping unknown class names to known class names.
    Raises:
      AssertionError: If we detect an internal error.
    """
    hierarchy = type_match.get_all_subclasses([self.ast, self.builtins])
    factory_protocols = type_match.TypeMatch(hierarchy)
    factory_partial = type_match.TypeMatch(hierarchy)
    solver_protocols = factory_protocols.solver
    solver_partial = factory_partial.solver

    unknown_classes = set()
    partial_classes = set()
    complete_classes = set()
    for cls in self.ast.classes:
      if is_unknown(cls):
        solver_protocols.register_variable(cls.name)
        solver_partial.register_variable(cls.name)
        unknown_classes.add(cls)
      elif is_partial(cls):
        partial_classes.add(cls)
      else:
        complete_classes.add(cls)

    protocol_classes_and_aliases = set(self.protocols.classes)
    for alias in self.protocols.aliases:
      if (not isinstance(alias.type, pytd.AnythingType)
          and alias.name != "protocols.Protocol"):
        protocol_classes_and_aliases.add(alias.type.cls)

    # solve equations from protocols first
    for protocol in protocol_classes_and_aliases:
      for unknown in unknown_classes:
        self.match_unknown_against_protocol(
            factory_protocols, solver_protocols, unknown, protocol)

    # also solve partial equations
    for complete in complete_classes.union(self.builtins.classes):
      for partial in partial_classes:
        if escape.unpack_partial(partial.name) == complete.name:
          self.match_partial_against_complete(
              factory_partial, solver_partial, partial, complete)

    partial_functions = set()
    complete_functions = set()
    for f in self.ast.functions:
      if is_partial(f):
        partial_functions.add(f)
      else:
        complete_functions.add(f)
    for partial in partial_functions:
      for complete in complete_functions.union(self.builtins.functions):
        if escape.unpack_partial(partial.name) == complete.name:
          self.match_call_record(
              factory_partial, solver_partial, partial, complete)

    log.info("=========== Equations to solve =============\n%s",
             solver_protocols)
    log.info("=========== Equations to solve (end) =======")
    solved_protocols = solver_protocols.solve()
    log.info("=========== Call trace equations to solve =============\n%s",
             solver_partial)
    log.info("=========== Call trace equations to solve (end) =======")
    solved_partial = solver_partial.solve()
    merged_solution = {}
    for unknown in itertools.chain(solved_protocols, solved_partial):
      if unknown in solved_protocols and unknown in solved_partial:
        merged_solution[unknown] = solved_protocols[unknown].union(
            solved_partial[unknown])
        # remove Any from set if present
        # if no restrictions are present, it will be labeled Any later
        # otherwise, Any will override other restrictions that were found
        merged_solution[unknown].discard("?")
      elif unknown in solved_protocols:
        merged_solution[unknown] = solved_protocols[unknown]
      else:
        merged_solution[unknown] = solved_partial[unknown]
    return merged_solution


def solve(ast, builtins_pytd, protocols_pytd):
  """Solve the unknowns in a pytd AST using the standard Python builtins.

  Args:
    ast: A pytd.TypeDeclUnit, containing classes named ~unknownXX.
    builtins_pytd: A pytd for builtins.
    protocols_pytd: A pytd for protocols.

  Returns:
    A tuple of (1) a dictionary (str->str) mapping unknown class names to known
    class names and (2) a pytd.TypeDeclUnit of the complete classes in ast.
  """
  builtins_pytd = transforms.RemoveMutableParameters(builtins_pytd)
  builtins_pytd = visitors.LookupClasses(builtins_pytd)
  protocols_pytd = visitors.LookupClasses(protocols_pytd)
  ast = visitors.LookupClasses(ast, builtins_pytd)
  return TypeSolver(
      ast, builtins_pytd, protocols_pytd).solve(), extract_local(ast)


def extract_local(ast):
  """Extract all classes that are not unknowns of call records of builtins."""
  return pytd.TypeDeclUnit(
      name=ast.name,
      classes=tuple(cls for cls in ast.classes if is_complete(cls)),
      functions=tuple(f for f in ast.functions if is_complete(f)),
      constants=tuple(c for c in ast.constants if is_complete(c)),
      type_params=ast.type_params,
      aliases=ast.aliases)


def convert_string_type(string_type, unknown, mapping, global_lookup, depth=0):
  """Convert a string representing a type back to a pytd type."""
  try:
    # Check whether this is a type declared in a pytd.
    cls = global_lookup.Lookup(string_type)
    base_type = pytd_utils.NamedOrClassType(cls.name, cls)
  except KeyError:
    # If we don't have a pytd for this type, it can't be a template.
    cls = None
    base_type = pytd_utils.NamedOrClassType(string_type, cls)

  if cls and cls.template:
    parameters = []
    for t in cls.template:
      type_param_name = unknown + "." + string_type + "." + t.name
      if type_param_name in mapping and depth < MAX_DEPTH:
        string_type_params = mapping[type_param_name]
        parameters.append(convert_string_type_list(
            string_type_params, unknown, mapping, global_lookup, depth + 1))
      else:
        parameters.append(pytd.AnythingType())
    return pytd.GenericType(base_type, tuple(parameters))
  else:
    return base_type


def convert_string_type_list(types_as_string, unknown, mapping,
                             global_lookup, depth=0):
  """Like convert_string_type, but operate on a list."""
  if not types_as_string or booleq.Solver.ANY_VALUE in types_as_string:
    # If we didn't find a solution for a type (the list of matches is empty)
    # then report it as "?", not as "nothing", because the latter is confusing.
    return pytd.AnythingType()
  return pytd_utils.JoinTypes(convert_string_type(type_as_string, unknown,
                                                  mapping, global_lookup, depth)
                              for type_as_string in types_as_string)


def insert_solution(result, mapping, global_lookup):
  """Replace ~unknown types in a pytd with the actual (solved) types."""
  subst = {
      unknown: convert_string_type_list(types_as_strings, unknown,
                                        mapping, global_lookup)
      for unknown, types_as_strings in mapping.items()}
  result = result.Visit(optimize.RenameUnknowns(subst))
  # We remove duplicates here (even though Optimize does so again) because
  # it's much faster before the string types are replaced.
  result = result.Visit(optimize.RemoveDuplicates())
  return result.Visit(visitors.ReplaceTypes(subst))


def convert_pytd(ast, builtins_pytd, protocols_pytd):
  """Convert pytd with unknowns (structural types) to one with nominal types."""
  builtins_pytd = builtins_pytd.Visit(visitors.ClassTypeToNamedType())
  mapping, result = solve(ast, builtins_pytd, protocols_pytd)
  log_info_mapping(mapping)
  lookup = pytd_utils.Concat(builtins_pytd, result)
  result = insert_solution(result, mapping, lookup)
  if log.isEnabledFor(logging.INFO):
    log.info("=========== solve result =============\n%s",
             pytd_utils.Print(result))
    log.info("=========== solve result (end) =============")
  return result


def log_info_mapping(mapping):
  """Print a raw type mapping. For debugging."""
  if log.isEnabledFor(logging.DEBUG):
    cutoff = 12
    log.debug("=========== (possible types) ===========")
    for unknown, possible_types in sorted(mapping.items()):
      assert isinstance(possible_types, (set, frozenset))
      if len(possible_types) > cutoff:
        log.debug("%s can be   %s, ... (total: %d)", unknown,
                  ", ".join(sorted(possible_types)[0:cutoff]),
                  len(possible_types))
      else:
        log.debug("%s can be %s", unknown,
                  ", ".join(sorted(possible_types)))
    log.debug("=========== (end of possible types) ===========")



from typing import Any

from oauthlib.oauth2.rfc6749.errors import InvalidRequestError as InvalidRequestError

from ..request_validator import RequestValidator as RequestValidator
from .base import GrantTypeBase as GrantTypeBase

log: Any

class HybridGrant(GrantTypeBase):
    request_validator: Any
    proxy_target: Any
    def __init__(self, request_validator: Any | None = ..., **kwargs) -> None: ...
    def add_id_token(self, token, token_handler, request): ...
    def openid_authorization_validator(self, request): ...



import codecs
import sys
from typing import Any, Callable, Text

# For convenience:
_Handler = Callable[[Exception], tuple[Text, int]]
_String = bytes | str
_Errors = str | Text | None
_Decodable = bytes | Text
_Encodable = bytes | Text

# This type is not exposed; it is defined in unicodeobject.c
class _EncodingMap(object):
    def size(self) -> int: ...

_MapT = dict[int, int] | _EncodingMap

def register(__search_function: Callable[[str], Any]) -> None: ...
def register_error(__errors: str | Text, __handler: _Handler) -> None: ...
def lookup(__encoding: str | Text) -> codecs.CodecInfo: ...
def lookup_error(__name: str | Text) -> _Handler: ...
def decode(obj: Any, encoding: str | Text = ..., errors: _Errors = ...) -> Any: ...
def encode(obj: Any, encoding: str | Text = ..., errors: _Errors = ...) -> Any: ...
def charmap_build(__map: Text) -> _MapT: ...
def ascii_decode(__data: _Decodable, __errors: _Errors = ...) -> tuple[Text, int]: ...
def ascii_encode(__str: _Encodable, __errors: _Errors = ...) -> tuple[bytes, int]: ...
def charbuffer_encode(__data: _Encodable, __errors: _Errors = ...) -> tuple[bytes, int]: ...
def charmap_decode(__data: _Decodable, __errors: _Errors = ..., __mapping: _MapT | None = ...) -> tuple[Text, int]: ...
def charmap_encode(__str: _Encodable, __errors: _Errors = ..., __mapping: _MapT | None = ...) -> tuple[bytes, int]: ...
def escape_decode(__data: _String, __errors: _Errors = ...) -> tuple[str, int]: ...
def escape_encode(__data: bytes, __errors: _Errors = ...) -> tuple[bytes, int]: ...
def latin_1_decode(__data: _Decodable, __errors: _Errors = ...) -> tuple[Text, int]: ...
def latin_1_encode(__str: _Encodable, __errors: _Errors = ...) -> tuple[bytes, int]: ...
def raw_unicode_escape_decode(__data: _String, __errors: _Errors = ...) -> tuple[Text, int]: ...
def raw_unicode_escape_encode(__str: _Encodable, __errors: _Errors = ...) -> tuple[bytes, int]: ...
def readbuffer_encode(__data: _String, __errors: _Errors = ...) -> tuple[bytes, int]: ...
def unicode_escape_decode(__data: _String, __errors: _Errors = ...) -> tuple[Text, int]: ...
def unicode_escape_encode(__str: _Encodable, __errors: _Errors = ...) -> tuple[bytes, int]: ...
def unicode_internal_decode(__obj: _String, __errors: _Errors = ...) -> tuple[Text, int]: ...
def unicode_internal_encode(__obj: _String, __errors: _Errors = ...) -> tuple[bytes, int]: ...
def utf_16_be_decode(__data: _Decodable, __errors: _Errors = ..., __final: int = ...) -> tuple[Text, int]: ...
def utf_16_be_encode(__str: _Encodable, __errors: _Errors = ...) -> tuple[bytes, int]: ...
def utf_16_decode(__data: _Decodable, __errors: _Errors = ..., __final: int = ...) -> tuple[Text, int]: ...
def utf_16_encode(__str: _Encodable, __errors: _Errors = ..., __byteorder: int = ...) -> tuple[bytes, int]: ...
def utf_16_ex_decode(
    __data: _Decodable, __errors: _Errors = ..., __byteorder: int = ..., __final: int = ...
) -> tuple[Text, int, int]: ...
def utf_16_le_decode(__data: _Decodable, __errors: _Errors = ..., __final: int = ...) -> tuple[Text, int]: ...
def utf_16_le_encode(__str: _Encodable, __errors: _Errors = ...) -> tuple[bytes, int]: ...
def utf_32_be_decode(__data: _Decodable, __errors: _Errors = ..., __final: int = ...) -> tuple[Text, int]: ...
def utf_32_be_encode(__str: _Encodable, __errors: _Errors = ...) -> tuple[bytes, int]: ...
def utf_32_decode(__data: _Decodable, __errors: _Errors = ..., __final: int = ...) -> tuple[Text, int]: ...
def utf_32_encode(__str: _Encodable, __errors: _Errors = ..., __byteorder: int = ...) -> tuple[bytes, int]: ...
def utf_32_ex_decode(
    __data: _Decodable, __errors: _Errors = ..., __byteorder: int = ..., __final: int = ...
) -> tuple[Text, int, int]: ...
def utf_32_le_decode(__data: _Decodable, __errors: _Errors = ..., __final: int = ...) -> tuple[Text, int]: ...
def utf_32_le_encode(__str: _Encodable, __errors: _Errors = ...) -> tuple[bytes, int]: ...
def utf_7_decode(__data: _Decodable, __errors: _Errors = ..., __final: int = ...) -> tuple[Text, int]: ...
def utf_7_encode(__str: _Encodable, __errors: _Errors = ...) -> tuple[bytes, int]: ...
def utf_8_decode(__data: _Decodable, __errors: _Errors = ..., __final: int = ...) -> tuple[Text, int]: ...
def utf_8_encode(__str: _Encodable, __errors: _Errors = ...) -> tuple[bytes, int]: ...

if sys.platform == "win32":
    def mbcs_decode(__data: _Decodable, __errors: _Errors = ..., __final: int = ...) -> tuple[Text, int]: ...
    def mbcs_encode(__str: _Encodable, __errors: _Errors = ...) -> tuple[bytes, int]: ...



from _typeshed import Self
from types import TracebackType
from typing import Any

from .tracer import Tracer

class SpanContext:
    EMPTY_BAGGAGE: dict[str, str]
    @property
    def baggage(self) -> dict[str, str]: ...

class Span:
    def __init__(self, tracer: Tracer, context: SpanContext) -> None: ...
    @property
    def context(self) -> SpanContext: ...
    @property
    def tracer(self) -> Tracer: ...
    def set_operation_name(self: Self, operation_name: str) -> Self: ...
    def finish(self, finish_time: float | None = ...) -> None: ...
    def set_tag(self: Self, key: str, value: str | bool | int | float) -> Self: ...
    def log_kv(self: Self, key_values: dict[str, Any], timestamp: float | None = ...) -> Self: ...
    def set_baggage_item(self: Self, key: str, value: str) -> Self: ...
    def get_baggage_item(self, key: str) -> str | None: ...
    def __enter__(self: Self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...
    def log_event(self: Self, event: Any, payload: Any | None = ...) -> Self: ...
    def log(self: Self, **kwargs: Any) -> Self: ...



from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class Index(Serialisable):
    tagname: str
    v: Any
    def __init__(self, v: int = ...) -> None: ...

class Tuple(Serialisable):  # type: ignore[misc]
    fld: Any
    hier: Any
    item: Any
    def __init__(self, fld: Any | None = ..., hier: Any | None = ..., item: Any | None = ...) -> None: ...

class TupleList(Serialisable):  # type: ignore[misc]
    c: Any
    tpl: Any
    __elements__: Any
    def __init__(self, c: Any | None = ..., tpl: Any | None = ...) -> None: ...

class Missing(Serialisable):
    tagname: str
    tpls: Any
    x: Any
    u: Any
    f: Any
    c: Any
    cp: Any
    bc: Any
    fc: Any
    i: Any
    un: Any
    st: Any
    b: Any
    __elements__: Any
    def __init__(
        self,
        tpls=...,
        x=...,
        u: Any | None = ...,
        f: Any | None = ...,
        c: Any | None = ...,
        cp: Any | None = ...,
        _in: Any | None = ...,
        bc: Any | None = ...,
        fc: Any | None = ...,
        i: Any | None = ...,
        un: Any | None = ...,
        st: Any | None = ...,
        b: Any | None = ...,
    ) -> None: ...

class Number(Serialisable):
    tagname: str
    tpls: Any
    x: Any
    v: Any
    u: Any
    f: Any
    c: Any
    cp: Any
    bc: Any
    fc: Any
    i: Any
    un: Any
    st: Any
    b: Any
    __elements__: Any
    def __init__(
        self,
        tpls=...,
        x=...,
        v: Any | None = ...,
        u: Any | None = ...,
        f: Any | None = ...,
        c: Any | None = ...,
        cp: Any | None = ...,
        _in: Any | None = ...,
        bc: Any | None = ...,
        fc: Any | None = ...,
        i: Any | None = ...,
        un: Any | None = ...,
        st: Any | None = ...,
        b: Any | None = ...,
    ) -> None: ...

class Error(Serialisable):
    tagname: str
    tpls: Any
    x: Any
    v: Any
    u: Any
    f: Any
    c: Any
    cp: Any
    bc: Any
    fc: Any
    i: Any
    un: Any
    st: Any
    b: Any
    __elements__: Any
    def __init__(
        self,
        tpls: Any | None = ...,
        x=...,
        v: Any | None = ...,
        u: Any | None = ...,
        f: Any | None = ...,
        c: Any | None = ...,
        cp: Any | None = ...,
        _in: Any | None = ...,
        bc: Any | None = ...,
        fc: Any | None = ...,
        i: Any | None = ...,
        un: Any | None = ...,
        st: Any | None = ...,
        b: Any | None = ...,
    ) -> None: ...

class Boolean(Serialisable):
    tagname: str
    x: Any
    v: Any
    u: Any
    f: Any
    c: Any
    cp: Any
    __elements__: Any
    def __init__(
        self, x=..., v: Any | None = ..., u: Any | None = ..., f: Any | None = ..., c: Any | None = ..., cp: Any | None = ...
    ) -> None: ...

class Text(Serialisable):
    tagname: str
    tpls: Any
    x: Any
    v: Any
    u: Any
    f: Any
    c: Any
    cp: Any
    bc: Any
    fc: Any
    i: Any
    un: Any
    st: Any
    b: Any
    __elements__: Any
    def __init__(
        self,
        tpls=...,
        x=...,
        v: Any | None = ...,
        u: Any | None = ...,
        f: Any | None = ...,
        c: Any | None = ...,
        cp: Any | None = ...,
        _in: Any | None = ...,
        bc: Any | None = ...,
        fc: Any | None = ...,
        i: Any | None = ...,
        un: Any | None = ...,
        st: Any | None = ...,
        b: Any | None = ...,
    ) -> None: ...

class DateTimeField(Serialisable):
    tagname: str
    x: Any
    v: Any
    u: Any
    f: Any
    c: Any
    cp: Any
    __elements__: Any
    def __init__(
        self, x=..., v: Any | None = ..., u: Any | None = ..., f: Any | None = ..., c: Any | None = ..., cp: Any | None = ...
    ) -> None: ...



import sys
from _typeshed import SupportsWrite
from collections.abc import Callable
from typing import Any, Generic, TypeVar
from typing_extensions import Literal

_T = TypeVar("_T")
_R_co = TypeVar("_R_co", covariant=True)
_FuncT = TypeVar("_FuncT", bound=Callable[..., Any])

# These definitions have special processing in mypy
class ABCMeta(type):
    __abstractmethods__: frozenset[str]
    def __init__(self, name: str, bases: tuple[type, ...], namespace: dict[str, Any]) -> None: ...
    def __instancecheck__(cls: ABCMeta, instance: Any) -> Any: ...
    def __subclasscheck__(cls: ABCMeta, subclass: Any) -> Any: ...
    def _dump_registry(cls: ABCMeta, file: SupportsWrite[str] | None = ...) -> None: ...
    def register(cls: ABCMeta, subclass: type[_T]) -> type[_T]: ...

def abstractmethod(funcobj: _FuncT) -> _FuncT: ...

class abstractclassmethod(classmethod[_R_co], Generic[_R_co]):
    __isabstractmethod__: Literal[True]
    def __init__(self: abstractclassmethod[_R_co], callable: Callable[..., _R_co]) -> None: ...

class abstractstaticmethod(staticmethod[_R_co], Generic[_R_co]):
    __isabstractmethod__: Literal[True]
    def __init__(self, callable: Callable[..., _R_co]) -> None: ...

class abstractproperty(property):
    __isabstractmethod__: Literal[True]

class ABC(metaclass=ABCMeta): ...

def get_cache_token() -> object: ...

if sys.version_info >= (3, 10):
    def update_abstractmethods(cls: type[_T]) -> type[_T]: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.well_known_types
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class Any(google.protobuf.message.Message, google.protobuf.internal.well_known_types.Any):
    """`Any` contains an arbitrary serialized protocol buffer message along with a
    URL that describes the type of the serialized message.

    Protobuf library provides support to pack/unpack Any values in the form
    of utility functions or additional generated methods of the Any type.

    Example 1: Pack and unpack a message in C++.

        Foo foo = ...;
        Any any;
        any.PackFrom(foo);
        ...
        if (any.UnpackTo(&foo)) {
          ...
        }

    Example 2: Pack and unpack a message in Java.

        Foo foo = ...;
        Any any = Any.pack(foo);
        ...
        if (any.is(Foo.class)) {
          foo = any.unpack(Foo.class);
        }

     Example 3: Pack and unpack a message in Python.

        foo = Foo(...)
        any = Any()
        any.Pack(foo)
        ...
        if any.Is(Foo.DESCRIPTOR):
          any.Unpack(foo)
          ...

     Example 4: Pack and unpack a message in Go

         foo := &pb.Foo{...}
         any, err := anypb.New(foo)
         if err != nil {
           ...
         }
         ...
         foo := &pb.Foo{}
         if err := any.UnmarshalTo(foo); err != nil {
           ...
         }

    The pack methods provided by protobuf library will by default use
    'type.googleapis.com/full.type.name' as the type URL and the unpack
    methods only use the fully qualified type name after the last '/'
    in the type URL, for example "foo.bar.com/x/y.z" will yield type
    name "y.z".


    JSON
    ====
    The JSON representation of an `Any` value uses the regular
    representation of the deserialized, embedded message, with an
    additional field `@type` which contains the type URL. Example:

        package google.profile;
        message Person {
          string first_name = 1;
          string last_name = 2;
        }

        {
          "@type": "type.googleapis.com/google.profile.Person",
          "firstName": <string>,
          "lastName": <string>
        }

    If the embedded message type is well-known and has a custom JSON
    representation, that representation will be embedded adding a field
    `value` which holds the custom JSON in addition to the `@type`
    field. Example (for message [google.protobuf.Duration][]):

        {
          "@type": "type.googleapis.com/google.protobuf.Duration",
          "value": "1.212s"
        }
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TYPE_URL_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    type_url: typing.Text
    """A URL/resource name that uniquely identifies the type of the serialized
    protocol buffer message. This string must contain at least
    one "/" character. The last segment of the URL's path must represent
    the fully qualified name of the type (as in
    `path/google.protobuf.Duration`). The name should be in a canonical form
    (e.g., leading "." is not accepted).

    In practice, teams usually precompile into the binary all types that they
    expect it to use in the context of Any. However, for URLs which use the
    scheme `http`, `https`, or no scheme, one can optionally set up a type
    server that maps type URLs to message definitions as follows:

    * If no scheme is provided, `https` is assumed.
    * An HTTP GET on the URL must yield a [google.protobuf.Type][]
      value in binary format, or produce an error.
    * Applications are allowed to cache lookup results based on the
      URL, or have them precompiled into a binary to avoid any
      lookup. Therefore, binary compatibility needs to be preserved
      on changes to types. (Use versioned type names to manage
      breaking changes.)

    Note: this functionality is not currently available in the official
    protobuf release, and it is not used for type URLs beginning with
    type.googleapis.com.

    Schemes other than `http`, `https` (or the empty scheme) might be
    used with implementation specific semantics.
    """

    value: builtins.bytes
    """Must be a valid serialized protocol buffer of the above specified type."""

    def __init__(self,
        *,
        type_url: typing.Optional[typing.Text] = ...,
        value: typing.Optional[builtins.bytes] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["type_url",b"type_url","value",b"value"]) -> None: ...
global___Any = Any



def get_unpatched(item): ...
def get_unpatched_class(cls): ...
def patch_all() -> None: ...
def patch_func(replacement, target_mod, func_name) -> None: ...
def get_unpatched_function(candidate): ...
def patch_for_msvc_specialized_compiler(): ...



from abc import abstractmethod
from typing import Any

from ._chart import ChartBase

class _AreaChartBase(ChartBase):
    grouping: Any
    varyColors: Any
    ser: Any
    dLbls: Any
    dataLabels: Any
    dropLines: Any
    __elements__: Any
    def __init__(
        self, grouping: str = ..., varyColors: Any | None = ..., ser=..., dLbls: Any | None = ..., dropLines: Any | None = ...
    ) -> None: ...
    @property
    @abstractmethod
    def tagname(self) -> str: ...

class AreaChart(_AreaChartBase):
    tagname: str
    grouping: Any
    varyColors: Any
    ser: Any
    dLbls: Any
    dropLines: Any
    x_axis: Any
    y_axis: Any
    extLst: Any
    __elements__: Any
    def __init__(self, axId: Any | None = ..., extLst: Any | None = ..., **kw) -> None: ...

class AreaChart3D(AreaChart):
    tagname: str
    grouping: Any
    varyColors: Any
    ser: Any
    dLbls: Any
    dropLines: Any
    gapDepth: Any
    x_axis: Any
    y_axis: Any
    z_axis: Any
    __elements__: Any
    def __init__(self, gapDepth: Any | None = ..., **kw) -> None: ...
